package ShowXPath;

import java.util.*;
import java.io.*;
import ShowXPath.StackNode.*;
import ShowXPath.GUI.*;
import ShowXPath.TreeNode.*;
import javax.swing.tree.DefaultMutableTreeNode;
import java.net.URL;
import java.text.DecimalFormat;

/*
 */
public class Context
{
	private static Main _main;
  private int _isPrepared = 0;
  private Stack _stack = new Stack();
  private LinkedList _subcontexts;
  private boolean _skipping = false;
  private String _skippingto;
  private LMark _mark = new LMark();
  private int _level = 0;
  private LTreeNode _context;
	private LRoot _root;
	private LNodeset _documentRoots;
	private LNodeset _backDocumentRoots;
	private int _axis;
	// private ContextFrame _contextFrame = new ContextFrame("SubContext");
	private int _position = 0;
	private int _totalNodes = 0;
	private LTreeNode _currentContext;
	private DefaultMutableTreeNode _treeNode;
	private String _stringName = "A Context";
	private ContextTree _contextTree;
	private Hashtable _variables;

  static public URL fileToURL(File file) {
    String path = file.getAbsolutePath();
    String fSep = System.getProperty("file.separator");
    if (fSep != null && fSep.length() == 1)
      path = path.replace(fSep.charAt(0), '/');
    if (path.length() > 0 && path.charAt(0) != '/')
      path = '/' + path;
    try {
      return new URL("file", null, path);
    }
    catch (java.net.MalformedURLException e) {
      /* According to the spec this could only happen if the file
				 protocol were not recognized. */
      throw new Error("unexpected MalformedURLException");
    }
  }

	public static void setMain(Main main)
	{
		_main = main;
	}
  
  public Context(LRoot root, ContextTree contextTree, LNodeset documentRoots,
								 Hashtable variables)
  {
		_variables = variables;
    _level = 0;
    _root = root;
		_documentRoots = documentRoots;
		_context = root;
		_contextTree = contextTree;
		//_contextFrame.setTree(_root);
  }

  private Context(int level, LRoot root, ContextTree contextTree,
									LNodeset documentRoots, Hashtable variables)
  {
		_variables = variables;
    _level = level;
    _root = root;
		_documentRoots = documentRoots;
		_context = root;
		_contextTree = contextTree;
		//_contextFrame.setTree(_root);
  }

	public void setContext(LTreeNode node, int position, int totalNodes, LTreeNode current, int axis)
	{
		_context = node;
		_position = position;
		_currentContext = current;
		_totalNodes = totalNodes;
		_axis = axis;
	}
	
  private void print(String string)
  {
    //System.out.println("In context: " + string);
  }

  private void printError(String string)
  {
    System.out.println("In context: Error: " + string);
  }

  public void ignore(String string)
  {
  }

  public void contextStart()
  {
    print("In contextStart");
  }

	public void startPath()
	{
		print("startPath");
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).startPath();
				}
				else
				{
					printError("startPath(): Expected a context from the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
			_backDocumentRoots = _documentRoots;
    }
  }
		
	public void endPath()
	{
		print("endPath");
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).endPath();
				}
				else
				{
					printError("endPath(): Expected a context from the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
			_documentRoots = _backDocumentRoots;
    }
  }
		

  public void mark()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).mark();
				}
				else
				{
					printError("mark(): Expected a context from the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      _stack.push(_mark);
      print("push mark");
    }
  }

	public void root()
	{
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).root();
				}
				else
				{
					print("Expected a context in root() in the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
			LNodeset set = new LNodeset();
			set.addSortAll(_documentRoots);
			_stack.push(set);
			print("root");
		}
	}

	public void context()
	{
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).context();
				}
				else
				{
					print("Expected a context in context() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
			LNodeset set = new LNodeset();
			set.addSort(_context);
			_stack.push(set);
			print("push Context: " + _context.toString());
		}
	}
	
  public void push(LStackNode node)
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).push(node);
				}
				else
				{
					print("Expected a context in push() the list of subcontexts");
				}
			}
    }
    else if (!_skipping)
    {
			_stack.push(node);
			print("push '" + node.toString() + "'");
    }
  }

  public void nodeset()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).nodeset();
				}
				else
				{
					print("Expected a context in nodeset() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode node = (LStackNode) _stack.pop();
			if (node instanceof LNodeset)
			{
				_stack.push(node);
			}
			else
			{
				LNodeset list = new LNodeset();
				list.add(node);
				_stack.push(list);
				print("push nodeset");
			}
    }
  }

  public LStackNode end()
  {
    if (_isPrepared > 0)
    {
      printError("end(): Nested contexts exist when end() is called");
    }
    else if (_skipping)
    {
      printError("end(): _Skipping past end.");
    }
    else if (_stack.size() > 1)
    {
      printError("end(): More than one item left on _stack.");
    }
    LStackNode nodeset = (LStackNode)_stack.pop();
		//_contextFrame.showNodes(nodeset);
		return nodeset;
  }

  public void iftrue(String label)
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).iftrue(label);
				}
				else
				{
					print("Expected a context in iftrue() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode node = (LStackNode) _stack.pop();
			LBoolean bool = node.toLBoolean();
			if (bool.getValue())
			{
				print("top of _stack is true, start _skipping to " + label);
				_skipping = true;
				_skippingto = label;
				_stack.push(bool);
			}
      print("if true");
    }
  }

	public void showContext()
	{
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).showContext();
				}
				else
				{
					print("Expected a context in showContext() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode node = (LStackNode) _stack.pop();
			_stack.push(node);
			//_contextFrame.showNodes(node);
      print("showcontext");
    }
	}

  public void pop()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).pop();
				}
				else
				{
					print("Expected a context in pop() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode node = (LStackNode) _stack.pop();
      print("pop");
    }
  }

  public void label(String l)
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).label(l);
				}
				else
				{
					print("Expected a context in label() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      print("label");
    }
    else if (l.equals(_skippingto))
    {
      _skipping = false;
    }
    else 
    {
      print("_Skipping label " + l);
    }
  }

  public void ancestor()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).ancestor();
				}
				else
				{
					print("Expected a context in ancestor() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode node = (LStackNode) _stack.pop();
			if (node instanceof LNodeset)
      {
        ListIterator i = ((LNodeset)node).listIterator();
        LNodeset newNodes = new LNodeset();
        while (i.hasNext())
        {
          LNodeset n = ((LTreeNode)i.next()).ancestor();
          newNodes.addSortAll(n);
        }
        print("Pushing nodes: " + newNodes.showNode());
        newNodes.removeDuplicates();
        _stack.push(newNodes);
      }
			else
			{
				print("Expecting LNodeset for ancestor(), but got " + node.type());
			}
      print("ancestor");
    }
  }

  public void attribute()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).attribute();
				}
				else
				{
					print("Expected a context in attribute() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode node = (LStackNode) _stack.pop();
			if (node instanceof LNodeset)
      {
        ListIterator i = ((LNodeset)node).listIterator();
        LNodeset newNodes = new LNodeset();
        while (i.hasNext())
        {
          LNodeset n = ((LTreeNode)i.next()).attribute();
          newNodes.addSortAll(n);
        }
        print("Pushing nodes: " + newNodes.showNode());
        newNodes.removeDuplicates();
        _stack.push(newNodes);
      }
			else
			{
				print("Expecting LNodeset for attribute(), but got " + node.type());
			}
      print("attribute");
    }
  }


  public void ancestor_or_self()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).ancestor_or_self();
				}
				else
				{
					print("Expected a context in ancestor_or_self() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode node = (LStackNode) _stack.pop();
			if (node instanceof LNodeset)
      {
        ListIterator i = ((LNodeset)node).listIterator();
        LNodeset newNodes = new LNodeset();
        while (i.hasNext())
        {
          LNodeset n = ((LTreeNode)i.next()).ancestor_or_self();
          newNodes.addSortAll(n);
        }
        print("Pushing nodes: " + newNodes.showNode());
        newNodes.removeDuplicates();
        _stack.push(newNodes);
      }
			else
			{
				print("Expecting LNodeset for ancestor_or_self(), but got " +
							node.type());
			}
      print("ancestor_or_self");
    }
  }

  public void child()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).child();
				}
				else
				{
					print("Expected a context in child() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode node = (LStackNode) _stack.pop();
			if (node instanceof LNodeset)
      {
        ListIterator i = ((LNodeset)node).listIterator();
        LNodeset newNodes = new LNodeset();
        while (i.hasNext())
        {
					LTreeNode tn = (LTreeNode)i.next();
          LNodeset n = tn.child();
          newNodes.addSortAll(n);
        }
        print("Pushing nodes: " + newNodes.showNode());
        newNodes.removeDuplicates();
        _stack.push(newNodes);
      }
			else
			{
				print("Expecting LNodeset for child(), but got " + node.type());
			}
      print("child");
    }
  }

  public void descendent()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).descendent();
				}
				else
				{
					print("Expected a context in descendent() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode node = (LStackNode) _stack.pop();
			if (node instanceof LNodeset)
      {
        ListIterator i = ((LNodeset)node).listIterator();
        LNodeset newNodes = new LNodeset();
        while (i.hasNext())
        {
          LNodeset n = ((LTreeNode)i.next()).descendent();
          newNodes.addSortAll(n);
        }
        print("Pushing nodes: " + newNodes.showNode());
        newNodes.removeDuplicates();
        _stack.push(newNodes);
      }
			else
			{
				print("Expecting LNodeset for descendent(), but got " + node.type());
			}
      print("descendent");
    }
  }

  public void descendent_or_self()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).descendent_or_self();
				}
				else
				{
					print("Expected a context in descendent_or_self() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode node = (LStackNode) _stack.pop();
			if (node instanceof LNodeset)
      {
        ListIterator i = ((LNodeset)node).listIterator();
        LNodeset newNodes = new LNodeset();
        while (i.hasNext())
        {
          LNodeset n = ((LTreeNode)i.next()).descendent_or_self();
          newNodes.addSortAll(n);
        }
        print("Pushing nodes: " + newNodes.showNode());
        newNodes.removeDuplicates();
        _stack.push(newNodes);
      }
			else
			{
				print("Expecting LNodeset for descendent_or_self(), but got " +
							node.type());
			}
      print("descendent_or_self");
    }
  }

  public void following()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).following();
				}
				else
				{
					print("Expected a context in following() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode node = (LStackNode) _stack.pop();

			if (node instanceof LNodeset)
      {
        ListIterator i = ((LNodeset)node).listIterator();
        LNodeset newNodes = new LNodeset();
        while (i.hasNext())
        {
          LNodeset n = ((LTreeNode)i.next()).following();
          newNodes.addSortAll(n);
        }
        print("Pushing nodes: " + newNodes.showNode());
        newNodes.removeDuplicates();
        _stack.push(newNodes);
      }
			else
			{
				print("Expecting LNodeset for following(), but got " + node.type());
			}
      print("following");
    }
  }

  public void following_sibling()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).following_sibling();
				}
				else
				{
					print("Expected a context in following_sibling() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode node = (LStackNode) _stack.pop();
			if (node instanceof LNodeset)
      {
        ListIterator i = ((LNodeset)node).listIterator();
        LNodeset newNodes = new LNodeset();
        while (i.hasNext())
        {
          LNodeset n = ((LTreeNode)i.next()).following_sibling();
          newNodes.addSortAll(n);
        }
        print("Pushing nodes: " + newNodes.showNode());
        newNodes.removeDuplicates();
        _stack.push(newNodes);
      }
			else
			{
				print("Expecting LNodeset for following_sibling(), but got " +
							node.type());
			}
      print("following_sibling");
    }
  }

  public void parent()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).parent();
				}
				else
				{
					print("Expected a context in parent() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode node = (LStackNode) _stack.pop();
      LNodeset newNodes = new LNodeset();
			if (node instanceof LNodeset)
      {
        ListIterator i = ((LNodeset)node).listIterator();
        while (i.hasNext())
        {
          LNodeset n = ((LTreeNode)i.next()).parent();
          newNodes.addSortAll(n);
        }
        newNodes.removeDuplicates();
        print("Pushing nodes: " + newNodes.showNode());
        _stack.push(newNodes);
      }
			else
			{
				print("Expecting LNodeset for parent(), but got " + node.type());
			}
      print("parent");
    }
  }
  
  public void preceding()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).preceding();
				}
				else
				{
					print("Expected a context in preceding() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode node = (LStackNode) _stack.pop();
			if (node instanceof LNodeset)
      {
        ListIterator i = ((LNodeset)node).listIterator();
        LNodeset newNodes = new LNodeset();
        while (i.hasNext())
        {
          LNodeset n = ((LTreeNode)i.next()).preceding();
          newNodes.addSortAll(n);
        }
        print("Pushing nodes: " + newNodes.showNode());
        newNodes.removeDuplicates();
        _stack.push(newNodes);
      }
			else
			{
				print("Expecting LNodeset for preceding(), but got " + node.type());
			}
      print("preceding");
    }
  }

  public void preceding_sibling()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).preceding_sibling();
				}
				else
				{
					print("Expected a context in preceding_sibling() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode node = (LStackNode) _stack.pop();

			if (node instanceof LNodeset)
      {
        ListIterator i = ((LNodeset)node).listIterator();
        LNodeset newNodes = new LNodeset();
        while (i.hasNext())
        {
          LNodeset n = ((LTreeNode)i.next()).preceding_sibling();
          newNodes.addSortAll(n);
        }
        print("Pushing nodes: " + newNodes.showNode());
        newNodes.removeDuplicates();
        _stack.push(newNodes);
      }
			else
			{
				print("Expecting LNodeset for preceding_sibling(), but got " +
							node.type());
			}
      print("preceding_sibling");
    }
  }
  
  public void self()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).self();
				}
				else
				{
					print("Expected a context in self() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode node = (LStackNode) _stack.pop();

			if (node instanceof LNodeset)
      {
        ListIterator i = ((LNodeset)node).listIterator();
        LNodeset newNodes = new LNodeset();
        while (i.hasNext())
        {
          LNodeset n = ((LTreeNode)i.next()).self();
          newNodes.addSortAll(n);
        }
        print("Pushing nodes: " + newNodes.showNode());
        newNodes.removeDuplicates();
        _stack.push(newNodes);
      }
			else
			{
				print("Expecting LNodeset for self(), but got " + node.type());
			}
      print("self");
    }
  }
  
  public void qname()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).qname();
				}
				else
				{
					print("Expected a context in qname() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode local = (LStackNode) _stack.pop();
      LStackNode space = (LStackNode) _stack.pop();
			if (local instanceof LString && space instanceof LString)
			{
				LQname name = new LQname(((LString)space).toString(),
																 ((LString)local).toString());
				_stack.push(name);
			}
			else if (!(local instanceof LString))
			{
				print("Expecting LString for qname 1st second argument but got " +
							local.type());
			}
			else if (!(space instanceof LString))
			{
				print("Expecting LString for qname 2nd second argument but got " +
							space.type());
			}
      print("qname");
    }
  }

  public void comment()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).comment();
				}
				else
				{
					print("Expected a context in comment() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode node = (LStackNode) _stack.pop();
			if (node instanceof LNodeset)
      {
        ListIterator i = ((LNodeset)node).listIterator();
        LNodeset newNodes = new LNodeset();
        while (i.hasNext())
        {
					LTreeNode tn = (LTreeNode)i.next();
					if (tn instanceof LComment)
					{
						newNodes.addSort(tn);
					}
        }
        print("Pushing nodes: " + newNodes.showNode());
        newNodes.removeDuplicates();
        _stack.push(newNodes);
      }
			else
			{
				print("Expecting LNodeset for comment(), but got " + node.type());
			}
      print("comment");
    }
  }

  public void nodetest()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).nodetest();
				}
				else
				{
					print("Expected a context in nodetest() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode test = (LStackNode) _stack.pop();
      LStackNode set = (LStackNode) _stack.pop();
			if (set instanceof LNodeset)
			{
				if (test instanceof LQname)
				{
					LQname name = (LQname)test;
					print("set is " + set.toString());
					print("qname = " + name.toString());
					print("qname.namespace = '" + name.getNamespace() + "'");
					print("qname.localName = '" + name.getLocalName() + "'");
					LNodeset nodes = (LNodeset)set;
					ListIterator i = nodes.listIterator();
					LNodeset newNodes = new LNodeset();
					print("Checking nodeset");
					while (i.hasNext())
					{
						LTreeNode n = (LTreeNode)i.next();
						print("element local_name= " + n.getLocal_name());
						print("element namespace= " + n.getNamespace());
						if (name.getLocalName().equals("*"))
						{
							if (name.getNamespace().equals("") &&
									(n instanceof LElement || n instanceof LAttribute))
							{
								newNodes.addSort(n);
							}
							else if (name.getNamespace().equals(n.getNamespace()) &&
											 (n instanceof LElement || n instanceof LAttribute))
							{
								newNodes.addSort(n);
							}
						}
						else if ((name.getLocalName().equals(n.getLocal_name()) &&
											name.getNamespace().equals(n.getNamespace())) &&
										 (n instanceof LElement || n instanceof LAttribute) )
						{
							newNodes.addSort(n);
						}
					}
					_stack.push(newNodes);
					print("new nodes: " + newNodes.showNode());
				}
				else
				{
					print("Expecting LQname in nodetest() but got " + test.type());
				}
      }
			else
			{
				print("Expecting LNodeset in nodetest() but got " + set.type());
			}
      print("nodetest");
    }
  }

  public void text()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).text();
				}
				else
				{
					print("Expected a context in text() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode node = (LStackNode) _stack.pop();
			if (node instanceof LNodeset)
      {
        ListIterator i = ((LNodeset)node).listIterator();
        LNodeset newNodes = new LNodeset();
        while (i.hasNext())
        {
					LTreeNode tn = (LTreeNode)i.next();
					if (tn instanceof LText)
					{
						newNodes.addSort(tn);
					}
        }
        print("Pushing nodes: " + newNodes.showNode());
        newNodes.removeDuplicates();
        _stack.push(newNodes);
      }
			else
			{
				print("Expecting LNodeset for text(), but got " + node.type());
			}
      print("text");
    }
  }

  public void processing_instruction()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).processing_instruction();
				}
				else
				{
					print("Expected a context in processing_instruction() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
			LStackNode li = (LStackNode) _stack.pop();
      LStackNode node = (LStackNode) _stack.pop();
			if (node instanceof LNodeset)
      {
        ListIterator i = ((LNodeset)node).listIterator();
        LNodeset newNodes = new LNodeset();
        while (i.hasNext())
        {
					LTreeNode tn = (LTreeNode)i.next();
					if (tn instanceof LProcessingInstruction)
					{
						if (li instanceof LString)
						{
							if (((LProcessingInstruction)tn).getName().equals(li.toString()))
							{
								newNodes.addSort(tn);
							}
						}
						else if (li instanceof LMark)
						{
							newNodes.addSort(tn);
						}
						else
						{
							print("Expecting LMark or LString for PI literal but got" +
										li.type());
						}
					}
        }
        print("Pushing nodes: " + newNodes.showNode());
        newNodes.removeDuplicates();
        _stack.push(newNodes);
      }
			else
			{
				print("Expecting LNodeset for procedding_instruction(), but got " +
							node.type());
			}
      print("processing_instruction");
    }
  }

  public void prepare()
  {
		print("Prepare " + (_isPrepared + 1));
    if (_isPrepared > 0)
    {
      _isPrepared++;
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).prepare();
				}
				else
				{
					print("Expected a context in prepare() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      _isPrepared++;
      LStackNode node = (LStackNode)_stack.pop();

			if (node instanceof LNodeset)
			{
				_subcontexts = new LinkedList();
				ListIterator i = ((LNodeset)node).listIterator();
				while (i.hasNext())
				{
					Context newContext = new Context(_root, _contextTree, _documentRoots,
																					 _variables);
					if (_treeNode != null)
					{
						_contextTree.addChild(_treeNode, newContext);
						print("**** Set up nn for next level ****");
					}
					newContext.setContext((LTreeNode)i.next(), i.nextIndex(),
																((LNodeset)node).size(), _currentContext, _axis);
					_subcontexts.addLast(newContext);
				}
				_stack.push(node);
			}
			else
			{
				print("Expecting LNodeset for prepare(), got " + node.type());
			}
    }
  }

  public void update()
  {
		print("Update " + (_isPrepared - 1));
    if (_isPrepared > 1)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).update();
				}
				else
				{
					print("Expected a context in update() the list of subcontexts");
				}
      }
			_isPrepared--;
    }
    else if (!_skipping)
    {
			LStackNode node = (LStackNode)_stack.pop();
			print("Updating...");
			if (node instanceof LNodeset)
			{
				ListIterator l = ((LNodeset)node).listIterator();
				ListIterator i = _subcontexts.listIterator();
				int index = 0;
				int size = ((LNodeset)node).size();
				while (i.hasNext())
				{
					index++;
					Context subContext = (Context)i.next();
					LStackNode nend = subContext.end();
					//subContext.disposeGUI();
					LTreeNode n = (LTreeNode) l.next();
					_contextTree.removeChild(subContext.getTreeNode());
					if (nend instanceof LNumber)
					{
						print("nend is a number. Run position() first.");
						Double d = new Double(((LNumber)nend).getValue());
						print("index is: " + index);
						print("ID is: " + n.getID());
						print("d is: " + d);
						if (_axis == Interpreter.FORWARD_AXIS)
						{
							if (index != d.intValue())
							{
								l.remove();
								print("Removing");
							}
						}
						else
						{
							if (size - index + 1 != d.intValue())
							{
								l.remove();
								print("Removing");
							}
						}
					}
					else
					{
						print("subtext before converting to boolean: " + nend.type());
						print("And value is: " + nend.toString());
						LBoolean b = nend.toLBoolean();
						if (!b.getValue())
						{
							l.remove();
							print("Removing");
						}
					}
				}
				_isPrepared--;
				print("**** Removed all children ****");
			}
			_stack.push(node);
    }
  }

  public void add()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).add();
				}
				else
				{
					print("Expected a context in add() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode op1 = (LStackNode) _stack.pop();
      LStackNode op2 = (LStackNode) _stack.pop();
      double r = op1.toLNumber().getValue() + op2.toLNumber().getValue();
      _stack.push(new LNumber(r));
      print("The result of addition: " + r);
      print("push add");
    }
  }

  public void sub()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).sub();
				}
				else
				{
					print("Expected a context in sub() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode op1 = (LStackNode) _stack.pop();
      LStackNode op2 = (LStackNode) _stack.pop();
      double r = op1.toLNumber().getValue() - op2.toLNumber().getValue();
      _stack.push(new LNumber(r));
      print("The result of subtraction: " + r);
      print("push sub");
    }
  }

  public void iffalse(String label)
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).iffalse(label);
				}
				else
				{
					print("Expected a context in iffalse() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
			LStackNode node = (LStackNode) _stack.pop();
			LBoolean bool = node.toLBoolean();
			if (!bool.getValue())
			{
				print("top of _stack is false, start _skipping to " + label);
				_skipping = true;
				_skippingto = label;
				_stack.push(bool);
			}
			else
			{
				print("top of _stack is true, not _skipping to " + label);
			}
      print("iffalse");
    }
  }

  public void equal()
  {
		print("equal");
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).equal();
				}
				else
				{
					print("Expected a context in equal() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode op2 = (LStackNode) _stack.pop();
      LStackNode op1 = (LStackNode) _stack.pop();
			boolean result = op1.equals(op2);
			_stack.push(new LBoolean(result));
		}
  }

  public void not()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).not();
				}
				else
				{
					print("Expected a context in not() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode node = (LStackNode) _stack.pop();
			if (node instanceof LBoolean)
			{
				if (((LBoolean)node).getValue())
				{
					_stack.push(new LBoolean(false));
				}
				else
				{
					_stack.push(new LBoolean(true));
				}
			}
			else
			{
				LBoolean n = node.toLBoolean();
				if (n.getValue())
				{
					_stack.push(new LBoolean(false));
				}
				else
				{
					_stack.push(new LBoolean(true));
				}
			}
      print("not");
    }
  }

  public void mul()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).mul();
				}
				else
				{
					print("Expected a context in mul() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode op1 = (LStackNode) _stack.pop();
      LStackNode op2 = (LStackNode) _stack.pop();
			double r = op1.toLNumber().getValue() * op2.toLNumber().getValue();
      _stack.push(new LNumber(r));
      print("The result of multiplication: " + r);
      print("push mul");
    }
  }

  public void div()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).div();
				}
				else
				{
					print("Expected a context in div() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode op1 = (LStackNode) _stack.pop();
      LStackNode op2 = (LStackNode) _stack.pop();
			double r = op1.toLNumber().getValue() / op2.toLNumber().getValue();
      _stack.push(new LNumber(r));
      print("The result of div: " + r);
      print("push div");
    }
  }

  public void mod()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).mod();
				}
				else
				{
					print("Expected a context in mod() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode op1 = (LStackNode) _stack.pop();
      LStackNode op2 = (LStackNode) _stack.pop();
			double r = op1.toLNumber().getValue() % op2.toLNumber().getValue();
      _stack.push(new LNumber(r));
      print("The result of mod: " + r);
      print("push mod");
    }
  }

  public void less()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).less();
				}
				else
				{
					print("Expected a context in less() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode op1 = (LStackNode) _stack.pop();
      LStackNode op2 = (LStackNode) _stack.pop();
			boolean b = op1.less(op2);
			_stack.push(new LBoolean(b));
      print("less");
    }
  }

  public void greater()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).greater();
				}
				else
				{
					print("Expected a context in greater() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode op1 = (LStackNode) _stack.pop();
      LStackNode op2 = (LStackNode) _stack.pop();
			boolean b = op1.greater(op2);
			_stack.push(new LBoolean(b));
      print("greater");
    }
  }

  public void less_equal()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).less_equal();
				}
				else
				{
					print("Expected a context in less_equal() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode op1 = (LStackNode) _stack.pop();
      LStackNode op2 = (LStackNode) _stack.pop();
			boolean b = op1.lessEquals(op2);
			_stack.push(new LBoolean(b));
      print("less_equal");
    }
  }

  public void greater_equal()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).greater_equal();
				}
				else
				{
					print("Expected a context in greater_equal() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode op1 = (LStackNode) _stack.pop();
      LStackNode op2 = (LStackNode) _stack.pop();
			boolean b = op1.greaterEquals(op2);
			_stack.push(new LBoolean(b));
      print("greater_equal");
    }
  }

  public void negate()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).negate();
				}
				else
				{
					print("Expected a context in negate() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode node = (LStackNode) _stack.pop();
			double r = -node.toLNumber().getValue();
			_stack.push(new LNumber(r));
      print("negate");
    }
  }

  public void union()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).union();
				}
				else
				{
					print("Expected a context in union() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode node1 = (LStackNode) _stack.pop();
      LStackNode node2 = (LStackNode) _stack.pop();
			LNodeset result = new LNodeset();
			if (node1 instanceof LNodeset)
			{
				result.addSortAll((LNodeset)node1);
			}
			else
			{
				print("Node 1 should be LNodeset for union");
			}
			if (node2 instanceof LNodeset)
			{
				result.addSortAll((LNodeset)node2);
			}
			else
			{
				print("Node 2 should be LNodeset for union");
			}
			result.removeDuplicates();
			_stack.push(result);
      print("union");
    }
  }

  public void value()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).value();
				}
				else
				{
					print("Expected a context in value() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode node = (LStackNode) _stack.pop();
			if (!(node instanceof LQname))
			{
				print("Value expects a LQname but got " + node.type());
			}
			else
			{
				print("node's value is: " + node.getStringValue());
				if (_variables.get(node.getStringValue()) != null)
				{
					print("variable has the value: " + ((LStackNode)_variables.get(node.getStringValue())).getStringValue());
					LStackNode snode = (LStackNode)_variables.get(node.getStringValue());
					_stack.push(snode);
				}
				else
				{
					print("variable is not found in symboltable");
					_stack.push(new LNodeset());
				}
			}
      print("value");
    }
  }

  public void call()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).call();
				}
				else
				{
					print("Expected a context in call() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
			LinkedList arguments = new LinkedList();
      LStackNode function = (LStackNode) _stack.pop();
			if (function instanceof LString)
			{
				String name = ((LString)function).toString();
				
				print("Function call on " + name);
				LStackNode argument;
				do {
					argument = (LStackNode) _stack.pop();
					if (!(argument instanceof LMark))
					{
						arguments.addFirst(argument);
					}
				} while (! ( argument instanceof LMark));
				String functionName = stripOpen(name);
				handleFunction(functionName, arguments);
			}
			else
			{
				print("Function name should be a LString but got " + function.type());
			}
      print("push call");
    }
  }

  public void namespace()
  {
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).namespace();
				}
				else
				{
					print("Expected a context in namespace() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
      LStackNode node = (LStackNode) _stack.pop();
			if (node instanceof LNodeset)
      {
        ListIterator i = ((LNodeset)node).listIterator();
        LNodeset newNodes = new LNodeset();
        while (i.hasNext())
        {
          LNodeset n = ((LTreeNode)i.next()).namespace();
          newNodes.addAll(n);
        }
        print("Pushing nodes: " + newNodes.showNode());
        newNodes.removeDuplicates();
        _stack.push(newNodes);
      }
			else
			{
				print("Expecting LNodeset for namespace(), but got " + node.type());
			}
      print("namespace");
    }
  }

	private String stripOpen(String string)
	{
		int pos = string.indexOf('(');
		return string.substring(0, pos).trim();
	}

	private boolean argumentSize(LinkedList args, int min, int max, String function)
	{
		if  (args.size() < min || args.size() > max)
		{
			print("Function " + function + "() has at least " + min + " arguments,");
			print("and at most " + max + "arguments. Instead " + args.size() + " arguments are given.");
			return false;
		}
		else
		{
			return true;
		}
	}

	private void handleFunction(String function, LinkedList arguments)
	{
		if (function.equals("true"))
		{
			argumentSize(arguments, 0, 0, "true");
			_stack.push(new LBoolean(true));
		}
		else if (function.equals("false"))
		{
			argumentSize(arguments, 0, 0, "false");
			_stack.push(new LBoolean(false));
		}
		else if (function.equals("unparsed-entity-uri"))
		{
			print("unparsed-entity-uri function is being handled");
			if (argumentSize(arguments, 1, 1, "unparsed-entity-uri"))
			{
				LStackNode node1 = (LStackNode) arguments.getFirst();
				LString sn = node1.toLString();
				LString s = new LString(_root.getUnparsedEntityURI(sn.getStringValue()));
				LNodeset result = new LNodeset();
				result.add(s);
				_stack.push(s);
			}
		}
		else if (function.equals("substring-before"))
		{
			print("substring-before function is being handled");
			if (argumentSize(arguments, 2, 2, "substring-before"))
			{
				LStackNode node1 = (LStackNode) arguments.getFirst();
				LStackNode node2 = (LStackNode) arguments.getLast();
				LString sn1 = node1.toLString();
				LString sn2 = node2.toLString();
				LString string;
				if (sn1.getStringValue().indexOf(sn2.getStringValue()) != -1)
				{
					String s = sn1.getStringValue();
					int index = s.indexOf(sn2.getStringValue());
					string = new LString(s.substring(0, index));
				}
				else
				{
					string = new LString("");
				}
				_stack.push(string);
			}
		}
		else if (function.equals("substring-after"))
		{
			print("substring-after function is being handled");
			if (argumentSize(arguments, 2, 2, "substring-after"))
			{
				LStackNode node1 = (LStackNode) arguments.getFirst();
				LStackNode node2 = (LStackNode) arguments.getLast();
				LString sn1 = node1.toLString();
				LString sn2 = node2.toLString();
				LString string;
				if (sn1.getStringValue().indexOf(sn2.getStringValue()) != -1)
				{
					String s = sn1.getStringValue();
					int index = s.indexOf(sn2.getStringValue());
					string = new LString(s.substring(index + sn2.getStringValue().length()));
				}
				else
				{
					string = new LString("");
				}
				_stack.push(string);
			}
		}
		else if (function.equals("normalize-space"))
		{
			print("normalize-space function is being handled");
			if (argumentSize(arguments, 0, 1, "normalize-space"))
			{
				String s;
				
				if (arguments.size() == 1)
				{
					LStackNode n = (LStackNode) arguments.getFirst();
					LString ss = n.toLString();
					s = ss.getStringValue();
				}
				else
				{
					s = _context.getStringValue();
				}
				s = s.trim();
				char[] c = s.toCharArray();
				char[] r = new char[s.length()];
				boolean isSpace = false;
				int j = 0;
				for (int i = 0; i < c.length; i++)
				{
					if (c[i] == '\t')
					{
						c[i] = ' ';
					}
					else if (c[i] == '\r')
					{
						c[i] = ' ';
					}
					else if (c[i] == '\n')
					{
						c[i] = ' ';
					}
					if (isSpace && c[i] == ' ')
					{
					}
					else
					{
						r[j] = c[i];
						j++;
						if (c[i] == ' ')
						{
							isSpace = true;
						}
						else
						{
							isSpace = false;
						}
					}
				}
				LString ls = new LString(new String(r, 0, j));
				_stack.push(ls);
			}
		}
		else if (function.equals("system-property"))
		{
			print("system-property function is being handled");
			if (argumentSize(arguments, 1, 1, "system-property"))
			{
				LStackNode node = (LStackNode)arguments.getFirst();
				LString sn = node.toLString();
				LQname qn = new LQname(sn.getStringValue());
				if (qn.getLocalName().equals("version") &&
						qn.getNamespace().equals("xsl"))
				{
					LString s = new LString("1.0");
					_stack.push(s);
				}
				else if (qn.getLocalName().equals("vendor") &&
						qn.getNamespace().equals("xsl"))
				{
					LString s = new LString("Khun Yee Fung");
					_stack.push(s);
				}
				else if (qn.getLocalName().equals("vendor-url") &&
						qn.getNamespace().equals("xsl"))
				{
					LString s = new LString("http://www.extend.com");
					_stack.push(s);
				}
				else
				{
					print("qn.localname is: " + qn.getLocalName());
					print("qn.namespace is: " + qn.getNamespace());
					LString s = new LString("");
					_stack.push(s);
				}
			}
		}
		else if (function.equals("string-length"))
		{
			print("string-length function is being handled");
			if (argumentSize(arguments, 1, 1, "string-length"))
			{
				LStackNode n = (LStackNode) arguments.getFirst();
				LString s;
				if (n instanceof LString)
				{
					s = (LString)n;
				}
				else
				{
					s = n.toLString();
				}
				LNumber num = new LNumber(s.toString().length());
				_stack.push(num);
			}
		}
		else if (function.equals("namespace-uri"))
		{
			print("namespace-uri function is being handled");
			if (argumentSize(arguments, 0, 1, "namespace-uri"))
			{
				if (arguments.size() == 1)
				{
					LStackNode node1 = (LStackNode)arguments.get(0);
					if (node1 instanceof LNodeset && ((LNodeset)node1).size() > 0)
					{
						LTreeNode tn = (LTreeNode)((LNodeset)node1).getFirst();
						LString s;
						if (tn instanceof LElement)
						{
							s = new LString(((LElement)tn).getNamespace_uri());
						}
						else
						{
							s = new LString("");
						}
						LNodeset result = new LNodeset();
						result.add(s);
						_stack.push(result);
					}
				}
				else
				{
					LString s;
					if (_context instanceof LElement)
					{
						s = new LString(((LElement)_context).getNamespace_uri());
					}
					else
					{
						s = new LString("");
					}
					LNodeset result = new LNodeset();
					result.add(s);
					_stack.push(result);
				}
			}
		}
		else if (function.equals("format-number"))
		{
			print("format-number function is being handled");
			if (argumentSize(arguments, 2, 3, "format-number"))
			{
				LStackNode node1 = (LStackNode) arguments.getFirst();
				LStackNode node2 = (LStackNode) arguments.get(1);
				LNumber n1 = node1.toLNumber();
				LString s2 = node2.toLString();
				if (arguments.size() == 2)
				{
					DecimalFormat df = new DecimalFormat(s2.getStringValue());
					LString result = new LString(df.format(n1.getValue()));
					_stack.push(result);
				}
			}
			else
			{
				_stack.push(new LString("error"));
			}
		}
		else if (function.equals("number"))
		{
			print("number function is being handled");
			if (argumentSize(arguments, 1, 1, "number"))
			{
				LStackNode n = (LStackNode) arguments.getFirst();
				LNumber num;
				if (n instanceof LNumber)
				{
					num = (LNumber)n;
				}
				else
				{
					num = n.toLNumber();
				}
				_stack.push(num);
			}
		}
		else if (function.equals("generate-id"))
		{
			print("generate-id function is being handled");
			if (argumentSize(arguments, 0, 1, "generate-id"))
			{
				if (arguments.size() == 1)
				{
					LStackNode sn = (LStackNode) arguments.getFirst();
					if (sn instanceof LNodeset)
					{
						LTreeNode tn = (LTreeNode) ((LNodeset)sn).getFirst();
						LString s = new LString("ID" + tn.getID());
					}
					else
					{
						print("The argument is not a nodeset");
						LString s = new LString("IDError");
					}
				}
				else
				{
					LString s = new LString("ID" + _currentContext.getID());
					_stack.push(s);
				}
			}
		}
		else if (function.equals("starts-with"))
		{
			print("starts-with function is being handled");
			if (argumentSize(arguments, 2, 2, "starts-with"))
			{
				LStackNode node1 = (LStackNode) arguments.getFirst();
				LStackNode node2 = (LStackNode) arguments.getLast();
				LString sn1 = node1.toLString();
				LString sn2 = node2.toLString();
				LBoolean bool;
				if (sn1.getStringValue().startsWith(sn2.getStringValue()))
				{
					bool = new LBoolean(true);
				}
				else
				{
					bool = new LBoolean(false);
				}
				_stack.push(bool);
			}
		}
		else if (function.equals("local-name"))
		{
			print("local-name function is being handled");
			if (argumentSize(arguments, 0, 1, "local-name"))
			{
				if (arguments.size() == 1)
				{
					LStackNode node = (LStackNode) arguments.getFirst();
					if (node instanceof LNodeset)
					{
						LTreeNode tn = (LTreeNode)((LNodeset)node).getFirst();
						LString s = new LString(tn.getLocal_name());
						_stack.push(s);
					}
				}
				else
				{
					LString s = new LString(_context.getLocal_name());
					_stack.push(s);
				}
			}
		}
		else if (function.equals("translate"))
		{
			print("translate function is being handled");
			if (argumentSize(arguments, 3, 3, "translate"))
			{
				LStackNode node1 = (LStackNode) arguments.getFirst();
				LStackNode node2 = (LStackNode) arguments.get(1);
				LStackNode node3 = (LStackNode) arguments.get(2);
				LString sn1 = node1.toLString();
				LString sn2 = node2.toLString();
				LString sn3 = node3.toLString();
				Hashtable ht = new Hashtable();
				char[] c1 = sn1.getStringValue().toCharArray();
				char[] c2 = sn2.getStringValue().toCharArray();
				char[] c3 = sn3.getStringValue().toCharArray();
				for (int i = 0; i < c2.length; i++)
				{
					Character ch = new Character(c2[i]);
					if (ht.get(ch) == null)
					{
						if (c3.length > i)
						{
							ht.put(ch, new String(c3, i, 1));
						}
						else
						{
							ht.put(ch, "");
						}
					}
				}
				int j = 0;
				char[] l = new char[c1.length];
				for (int i = 0; i < c1.length; i++)
				{
					Character ch = new Character(c1[i]);
					if (ht.get(ch) != null)
					{
						if (!((String)ht.get(ch)).equals(""))
						{
							l[j] = ((String)ht.get(ch)).charAt(0);
							j++;
						}
					}
					else
					{
						l[j] = c1[i];
						j++;
					}
				}
				LString s = new LString(new String(l, 0, j));
				_stack.push(s);
			}
						
		}
		else if (function.equals("substring"))
		{
			print("substring function is being handled");
			if (argumentSize(arguments, 2, 3, "substring"))
			{
				LStackNode node1 = (LStackNode) arguments.getFirst();
				LStackNode node2 = (LStackNode) arguments.get(1);
				LString sn1 = node1.toLString();
				int sn2 = (int)node2.toLNumber().getValue() - 1;
				int sn3;
				if (arguments.size() == 3)
				{
					LStackNode node3 = (LStackNode) arguments.get(2);
					sn3 = (int)(node3.toLNumber().getValue() + sn2);
				}
				else
				{
					sn3 = sn1.getStringValue().length();
				}
				LString string = new LString(sn1.getStringValue().substring(sn2, sn3));
				_stack.push(string);
			}
		}
		else if (function.equals("document"))
		{
			print("document function is being handled");
			if (argumentSize(arguments, 1, 2, "document"))
			{
				LStackNode node1 = (LStackNode) arguments.get(0);
				LNodeset result = new LNodeset();
				_documentRoots = new LNodeset();
				if (arguments.size() == 1)
				{
					if (node1 instanceof LNodeset)
					{
						ListIterator i = ((LNodeset)node1).listIterator();
						print("UNION [");
						while (i.hasNext())
						{
							LTreeNode e = (LTreeNode)i.next();
							URL baseU;
							URL newU;
							try
							{
								baseU = new URL(e.getURI());
								newU = new URL(baseU, e.getStringValue());
							}
							catch (java.net.MalformedURLException a)
							{
								print(a.toString());
								baseU = null;
								newU = null;
							}
							print("read_document(" + e.getStringValue() +
										"," + e.getURI() + ")");
							print("newU is: " + newU);
							LRoot root = _main.setDocument(newU);
							if (root != null)
							{
								LNodeset nroot = new LNodeset();
								nroot.add(root);
								result.addSortAll(nroot);
								_documentRoots.add(root);
								print("adding root to document roots: " + root.toString());
							}
						}
						print("]");
						result.removeDuplicates();
					}
					else
					{
						URL newU;
						try
						{
							newU = new URL(node1.getStringValue());
						}
						catch (java.net.MalformedURLException a)
						{
							print(a.toString());
							newU = fileToURL(new File(node1.getStringValue()));
						}
						print("New U is: " + newU);
						print("read_document(" + node1.getStringValue() +
									", null)");
						if (newU != null)
						{
							LRoot root = _main.setDocument(newU);
							if (root != null)
							{
								result.add(root);
								_documentRoots.add(root);
								print("adding root to document roots: " + root.toString());
							}
						}
					}
				}
				else
				{
					LStackNode node2 = (LStackNode) arguments.get(1);
					String baseURI;
					if (node2 instanceof LNodeset)
					{
						if (((LNodeset)node2).size() != 0)
						{
							baseURI = ((LTreeNode)((LNodeset)node2).getFirst()).getURI();
						}
						else
						{
							baseURI = null;
						}
					}
					else
					{
						baseURI = null;
					}
					if (node1 instanceof LNodeset)
					{
						ListIterator i = ((LNodeset)node1).listIterator();
						print("UNION [");
						while (i.hasNext())
						{
							LTreeNode e = (LTreeNode)i.next();
							URL baseU;
							URL newU;
							try
							{
								baseU = new URL(baseURI);
								newU = new URL(baseU, e.getStringValue());
							}
							catch (java.net.MalformedURLException a)
							{
								print(a.toString());
								baseU = null;
								newU = null;
							}
							print("new U: " + newU);
							print("read_document(" + e.getStringValue() +
										"," + baseURI + ")");
							LRoot root = _main.setDocument(newU);
							if (root != null)
							{
								LNodeset r = new LNodeset();
								result.addSortAll(r);
								_documentRoots.add(root);
								print("adding root to document roots: " + root.toString());
							}
						}
						result.removeDuplicates();
						print("]");
					}
					else
					{
						URL baseU;
						URL newU;
						try
						{
							baseU = new URL(baseURI);
							newU = new URL(baseU, node1.getStringValue());
						}
						catch (java.net.MalformedURLException a)
						{
							print(a.toString());
							baseU = null;
							newU = null;
						}
						print("New U is: " + newU);
						print("read_document(" + node1.getStringValue() +
									"," + baseURI + ")");
						LRoot root = _main.setDocument(newU);
						if (root != null)
						{
							result.add(root);
							print("adding root to document roots: " + root.toString());
							_documentRoots.add(root);
						}
					}
				}
				_stack.push(result);
			}
		}
		else if (function.equals("contains"))
		{
			print("contains function is being handled");
			if (argumentSize(arguments, 2, 2, "contains"))
			{
				LStackNode node1 = (LStackNode) arguments.getFirst();
				LStackNode node2 = (LStackNode) arguments.getLast();
				LString sn1 = node1.toLString();
				LString sn2 = node2.toLString();
				LBoolean bool;
				if (sn1.getStringValue().indexOf(sn2.getStringValue()) != -1)
				{
					bool = new LBoolean(true);
				}
				else
				{
					bool = new LBoolean(false);
				}
				_stack.push(bool);
			}
		}
		else if (function.equals("position"))
		{
			print("position function is being handled");
			if (argumentSize(arguments, 0, 0, "position"))
			{
				if (_axis == Interpreter.FORWARD_AXIS)
				{
					print("The axis is a forward axis");
					LNumber num = new LNumber(_position);
					_stack.push(num);
				}
				else
				{
					print("The axis is a reverse axis");
					LNumber num = new LNumber(_totalNodes - _position + 1);
					_stack.push(num);
				}
			}
		}
		else if (function.equals("ceiling"))
		{
			print("ceiling function is being handled");
			if (argumentSize(arguments, 1, 1, "ceiling"))
			{
				LStackNode node = (LStackNode)arguments.getFirst();
				LNumber nn = node.toLNumber();
				LNumber rn = new LNumber(Math.ceil(nn.getValue()));
				_stack.push(rn);
			}
		}
		else if (function.equals("current"))
		{
			print("current function is being handled");
			if (argumentSize(arguments, 0, 0, "current"))
			{
				LNodeset l = new LNodeset();
				l.add(_currentContext);
				_stack.push(l);
			}
		}
		else if (function.equals("boolean"))
		{
			print("boolean function is being handled");
			if (argumentSize(arguments, 1, 1, "boolean"))
			{
				LStackNode n = (LStackNode) arguments.getFirst();
				LBoolean bool;
				if (n instanceof LBoolean)
				{
					bool = (LBoolean)n;
				}
				else
				{
					bool = n.toLBoolean();
				}
				_stack.push(bool);
			}
		}
		else if (function.equals("string"))
		{
			print("string function is being handled");
			if (argumentSize(arguments, 1, 1, "string"))
			{
				LStackNode n = (LStackNode) arguments.getFirst();
				LString string;
				if (n instanceof LString)
				{
					string = (LString)n;
				}
				else
				{
					string = n.toLString();
				}
				_stack.push(string);
			}
		}
		else if (function.equals("concat"))
		{
			print("concat function is being handled");
			if (argumentSize(arguments, 2, arguments.size() + 1, "concat"))
			{
				ListIterator i = arguments.listIterator();
				LString s = new LString("");
				while (i.hasNext())
				{
					LStackNode n = (LStackNode) i.next();
					LString sn = n.toLString();
					s.concat(sn);
				}
				_stack.push(s);
			}
		}
		else if (function.equals("count"))
		{
			print("count function is being handled");
			if (argumentSize(arguments, 1, 1, "count"))
			{
				LStackNode sn = (LStackNode)arguments.getFirst();
				if (sn instanceof LNodeset)
				{
					LNumber num = new LNumber(((LNodeset)sn).size());
					_stack.push(num);
				}
				else
				{
					print("The argument of count must be a nodeset.");
					LNumber num = new LNumber(0);
					_stack.push(num);
				}
			}
		}
		else if (function.equals("round"))
		{
			print("round function is being handled");
			if (argumentSize(arguments, 1, 1, "round"))
			{
				LStackNode node = (LStackNode)arguments.getFirst();
				LNumber nn = node.toLNumber();
				LNumber rn = new LNumber((double)Math.round(nn.getValue()));
				_stack.push(rn);
			}
		}
		else if (function.equals("floor"))
		{
			print("floor function is being handled");
			if (argumentSize(arguments, 1, 1, "floor"))
			{
				LStackNode node = (LStackNode)arguments.getFirst();
				LNumber nn = node.toLNumber();
				LNumber rn = new LNumber(Math.floor(nn.getValue()));
				_stack.push(rn);
			}
		}
		else if (function.equals("last"))
		{
			print("last function is being handled");
			if (argumentSize(arguments, 0, 0, "last"))
			{
				LNumber num = new LNumber(_totalNodes);
				_stack.push(num);
			}
		}
		else if (function.equals("name"))
		{
			print("name function is being handled");
			if (argumentSize(arguments, 0, 1, "name"))
			{
				if (arguments.size() == 1)
				{
					LStackNode node = (LStackNode) arguments.getFirst();
					if (node instanceof LNodeset)
					{
						LTreeNode tn = (LTreeNode)((LNodeset)node).getFirst();
						LString s = new LString(tn.getName());
						_stack.push(s);
					}
				}
				else
				{
					LString s = new LString(_context.getName());
					_stack.push(s);
				}
			}
		}
		else if (function.equals("lang"))
		{
			print("lang function is being handled");
			if (argumentSize(arguments, 1, 1, "lang"))
			{
				LStackNode node1 = (LStackNode) arguments.getFirst();
				LString sn = node1.toLString();
				String snn = sn.getStringValue();
				String lang = _context.getLang();
				LBoolean b;
				if (lang.equals(""))
				{
					b = new LBoolean(false);
				}
				else if (lang.equalsIgnoreCase(snn))
				{
					b = new LBoolean(true);
				}
				else if (lang.indexOf('-') != -1)
				{
					if (lang.substring(0, lang.indexOf('-')).equalsIgnoreCase(snn))
					{
						b = new LBoolean(true);
					}
					else
					{
						b = new LBoolean(false);
					}
				}
				else
				{
					b = new LBoolean(false);
				}
				_stack.push(b);
			}
		}
		else if (function.equals("key"))
		{
			print("key function is being handled");
			_stack.push(new LNodeset());
		}
		else if (function.equals("function-available"))
		{
			print("function-available function is being handled");
			if (argumentSize(arguments, 1, 1, "function-available"))
			{
				LBoolean b = new LBoolean(false);
				_stack.push(b);
			}
		}
		else if (function.equals("element-available"))
		{
			print("element-variable function is being handled");
			if (argumentSize(arguments, 1, 1, "element-available"))
			{
				LBoolean b = new LBoolean(false);
				_stack.push(b);
			}
		}
		else if (function.equals("sum"))
		{
			print("sum function is being handled");
			if (argumentSize(arguments, 1, 1, "sum"))
			{
				LStackNode node = (LStackNode)arguments.getFirst();
				if (node instanceof LNodeset)
				{
					LNodeset ns = (LNodeset)node;
					ListIterator i = ns.listIterator();
					double d = 0.0;
					while (i.hasNext())
					{
						LNumber n = new LString(((LTreeNode)i.next()).getStringValue()).toLNumber();
						d = d + n.getValue();
					}
					LNumber rn = new LNumber(d);
					_stack.push(rn);
				}
			}
		}
		else if (function.equals("id"))
		{
			print("id function is being handled");
			if (argumentSize(arguments, 1, 1, "id"))
			{
				LStackNode node = (LStackNode) arguments.getFirst();
				LNodeset result = new LNodeset();
				if (node instanceof LNodeset)
				{
					ListIterator i = ((LNodeset)node).listIterator();
					while (i.hasNext())
					{
						String idList = ((LTreeNode)i.next()).getStringValue();
						StringTokenizer st = new StringTokenizer(idList, " \t\r\n");
						while (st.hasMoreElements())
						{
							String token = st.nextToken();
							ListIterator j = _documentRoots.listIterator();
							while (j.hasNext())
							{
								LRoot r = (LRoot)j.next();
								if (r.getElementID(token) != null)
								{
									result.addSort(r.getElementID(token));
									print("node selected by id is: " + r.getElementID(token).toString());
								}
							}
						}
					}
				}
				else
				{
					String idList = node.getStringValue();
					StringTokenizer st = new StringTokenizer(idList, " \t\r\n");
					while (st.hasMoreElements())
					{
						String token = st.nextToken();
						ListIterator j = _documentRoots.listIterator();
						while (j.hasNext())
						{
							LRoot r = (LRoot)j.next();
							if (r.getElementID(token) != null)
							{
								result.addSort(r.getElementID(token));
								print("node selected by id is: " + r.getElementID(token).toString());
							}
						}
					}
				}
				result.removeDuplicates();
				_stack.push(result);
			}
		}
		else if (function.equals("not"))
		{
			print("not function is being handled");
			if (argumentSize(arguments, 1, 1, "not"))
			{
				LStackNode node = (LStackNode) arguments.getFirst();
				LBoolean bn = node.toLBoolean();
				LBoolean br;
				if (bn.getValue())
				{
					br = new LBoolean(false);
				}
				else
				{
					br = new LBoolean(true);
				}
				_stack.push(br);
			}
		}
		else
		{
			print("A function not known: " + function);
		}
	}

	public void setForwardAxis()
	{
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).setForwardAxis();
				}
				else
				{
					print("Expected a context in mark() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
			_axis = Interpreter.FORWARD_AXIS;
			print("AXIS is now set to FORWARD");
		}
	}

	public void setReverseAxis()
	{
    if (_isPrepared > 0)
    {
      ListIterator i = _subcontexts.listIterator();
      while (i.hasNext())
      {
				Object c = i.next();
				if (c instanceof Context)
				{
					((Context)c).setReverseAxis();
				}
				else
				{
					print("Expected a context in mark() the list of subcontexts");
				}
      }
    }
    else if (!_skipping)
    {
			_axis = Interpreter.REVERSE_AXIS;
			print("AXIS is now set to reverse");
		}
	}

	public void setTreeNode(DefaultMutableTreeNode treeNode)
	{
		_treeNode = treeNode;
	}

	public DefaultMutableTreeNode getTreeNode()
	{
		return _treeNode;
	}

	public void setName(String name)
	{
		_stringName = name;
	}

	public String toString()
	{
		String type = "";
		if (_context instanceof LRoot)
		{
			type = "ROOT:";
		}
		else if (_context instanceof LElement)
		{
			type = "ELEMENT:";
		}
		else if (_context instanceof LAttribute)
		{
			type = "ATTRIBUTE:";
		}
		else if (_context instanceof LComment)
		{
			type = "COMMENT:";
		}
		else if (_context instanceof LText)
		{
			type = "TEXT:";
		}
		else if (_context instanceof LProcessingInstruction)
		{
			type = "PROCESSING INSTRUCTION:";
		}
		else if (_context instanceof LNamespace)
		{
			type = "XMLNS:";
		}
		type = type + "[" + _context.getID() + "] = ";
		String theString;
		theString = type + _context.toString();
		return theString;
	}

	public LStackNode stackRoot()
	{
		if (_stack.size() > 0)
		{
			print("stackRoot: Stack is not empty and returning the first element");
			return (LStackNode)_stack.get(0);
		}
		else
		{
			print("stackRoot: Stack is empty and not returning anything");
			return null;
		}
	}

	public Stack getStack()
	{
		return _stack;
	}
}
