package ShowXPath;

import java.util.*;
import java.io.*;

import ShowXPath.StackNode.*;
import ShowXPath.TreeNode.*;

import java.io.*;
import java.util.*;

/*
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import com.ibm.xml.parsers.NonValidatingDOMParser;
*/

import ShowXPath.XMLParser.*;
import ShowXPath.*;
import ShowXPath.GUI.*;

public class Executor
{
	private Context _context = null;
	private ContextTree _contextTree = null;
	private LRoot _treeRoot;
	private Main _main;
	private String _expr = "";
	private Thread _thread = null;
	private boolean _stepBig = false;
	private boolean _stepSmall = false;
	private boolean _stepBaby = false;
	private Hashtable _variables;
	
	public Executor(String uri, Main main, Hashtable variables)
	{
		_main = main;
		_variables = variables;
		// DOM2Internal dom = new DOM2Internal(uri);
		// LRoot root = dom.convert();
		LTreeNode.resetID();
		SaxTree saxTree = new SaxTree(uri);
		LRoot root = saxTree.getRoot();

		LNodeset list = root.descendent();
		LTreeNode firstChild = (LTreeNode) list.getFirst();
		_treeRoot = root;
		LNodeset roots = new LNodeset();
		roots.add(root);
		_context = new Context(root, main.getContextTree(), roots, _variables);
		_main.setRootContextDisplay(_context);
	}

	public Executor(LRoot root, Main main, Hashtable variables)
	{
		_variables = variables;
		_main = main;
		LNodeset list = root.descendent();
		LTreeNode firstChild = (LTreeNode) list.getFirst();
		_treeRoot = root;
		LNodeset roots = new LNodeset();
		roots.add(root);
		_context = new Context(root, main.getContextTree(), roots, _variables);
		_main.setRootContextDisplay(_context);
		_treeRoot = root;
	}

	public void setContext(LTreeNode n)
	{
		_context.setContext(n, 1, 1, n, Interpreter.FORWARD_AXIS);
	}

	public void setContextTree(ContextTree contextTree)
	{
		_contextTree = contextTree;
		_contextTree.addRoot(_context);
	}

	private void print(String string)
	{
		//System.out.println("In executor: " + string);
	}

	public void ignore(String string)
	{
		print(string);
	}

	public void setThread(Thread thread)
	{
		_thread = thread;
	}

	public void executorStart()
	{
		print("executionStart");
		_context.contextStart();
	}

	public void cleanUp()
	{
		if (_context != null && _contextTree != null)
		{
			_contextTree.removeAll();
		}
	}

	public void mark()
	{
		print("mark");
		_context.mark();
	}

	public void root()
	{
		_context.root();
	}

	public void context()
	{
		_context.context();
	}

	public void startPath()
	{
		_context.startPath();
	}

	public void endPath()
	{
		_context.endPath();
	}

	public void pushLiteral(String node)
	{
		if ((node.startsWith("'") && node.endsWith("'")) ||
				(node.startsWith("\"") && node.endsWith("\"")))
		{
			_context.push(new LString(node.substring(0, node.length()-1).substring(1)));
		}
		else
		{
			_context.push(new LString(node));
		}
	}

	public void pushNumber(String node)
	{
		_context.push(new LNumber(node));
		print("push " + node);
	}

	public void nodeset()
	{
		print("nodeset");
		_context.nodeset();
	}

	public void end()
	{
		print("end");
		LStackNode node = _context.end();
		//_context.disposeGUI();
		_main.showNodes(node);
		if (node instanceof LString)
		{
			print("Datatype: String");
		}
		else if (node instanceof LBoolean)
		{
			print("Datatype: Boolean");
		}
		else if (node instanceof LNumber)
		{
			print("Datatype: Number");
		}
		else if (node instanceof LNodeset)
		{
			print("Datatype: Nodeset");
		}
		print("End of expression: " + node.getStringValue());
		print("Expression: " + _expr);
	}

	public void iftrue(String label)
	{
		print("iftrue " + label);
		_context.iftrue(label);
	}

	public void pop()
	{
		print("pop");
		_context.pop();
	}

	public void label(String l)
	{
		print(l + ":");
		_context.label(l);
	}

	public void ancestor()
	{
		print("ancestor");
		_context.ancestor();
	}

	public void attribute()
	{
		print("atribute");
		_context.attribute();
	}


	public void ancestor_or_self()
	{
		print("ancestor_or_self");
		_context.ancestor_or_self();
	}

	public void child()
	{
		print("child");
		_context.child();
	}

	public void descendent()
	{
		print("descendent");
		_context.descendent();
	}

	public void descendent_or_self()
	{
		print("descendent_or_self");
		_context.descendent_or_self();
	}

	public void following()
	{
		print("following");
		_context.following();
	}

	public void following_sibling()
	{
		print("following_sibling");
		_context.following_sibling();
	}

	public void parent()
	{
		print("parent");
		_context.parent();
	}
	
	public void preceding()
	{
		print("preceding");
		_context.preceding();
	}

	public void preceding_sibling()
	{
		print("preceding_sibling");
		_context.preceding_sibling();
	}
	
	public void self()
	{
		print("self");
		_context.self();
	}
	
	public void qname()
	{
		print("qname");
		_context.qname();
	}

	public void comment()
	{
		print("comment");
		_context.comment();
	}

	public void nodetest()
	{
		print("nodetest");
		_context.nodetest();
	}

	public void text()
	{
		print("text");
		_context.text();
	}

	public void processing_instruction()
	{
		print("processing_instruction");
		_context.processing_instruction();
	}

	public void prepare()
	{
		print("prepare");
		_context.prepare();
	}

	public void update()
	{
		print("update");
		_context.update();
	}

	public void add()
	{
		print("add");
		_context.add();
	}

	public void sub()
	{
		print("sub");
		_context.sub();
	}

	public void iffalse(String label)
	{
		print("iffalse " + label);
		_context.iffalse(label);
	}

	public void equal()
	{
		print("equal");
		_context.equal();
	}

	public void not()
	{
		print("not");
		_context.not();
	}

	public void mul()
	{
		print("mul");
		_context.mul();
	}

	public void div()
	{
		print("div");
		_context.div();
	}

	public void mod()
	{
		print("mod");
		_context.mod();
	}

	public void less()
	{
		print("less");
		_context.less();
	}

	public void greater()
	{
		print("greater");
		_context.greater();
	}

	public void less_equal()
	{
		print("less_equal");
		_context.less_equal();
	}

	public void greater_equal()
	{
		print("greater_equal");
		_context.greater_equal();
	}

	public void negate()
	{
		print("negate");
		_context.negate();
	}

	public void union()
	{
		print("union");
		_context.union();
	}

	public void value()
	{
		print("value");
		_context.value();
	}

	public void call()
	{
		print("call");
		_context.call();
	}

	public void namespace()
	{
		print("namespace");
		_context.namespace();
	}

	public void setForwardAxis()
	{
		_context.setForwardAxis();
	}

	public void setReverseAxis()
	{
		_context.setReverseAxis();
	}

	public void noStep(String string)
	{
		if (string == null)
		{
			_expr = _expr + " -NO BAD- ";
		}
		else
		{
			_expr = _expr + " -" + string + "- ";
		}
		_main.setStep(_expr);
	}

	public void babyStep(String string)
	{
		if (string == null)
		{
			_expr = _expr + " -BABY BAD- ";
		}
		else
		{
			_expr = _expr + " -" + string + "- ";
		}
		_main.setStep(_expr);
		if (_stepBaby)
		{
			_main.displayContext();
			synchronized(_thread)
			{
				try
				{
					_thread.wait();
				}
				catch (Exception e)
				{
				}
			}
		}
	}
	
	public void smallStep(String string)
	{
		if (string == null)
		{
			_expr = _expr + " -SMALL BAD- ";
		}
		else
		{
			_expr = _expr + " -" + string + "- ";
		}
		_main.setStep(_expr);
    if (_stepSmall)
		{
			_main.displayContext();
			synchronized(_thread)
			{
				try
				{
					_thread.wait();
				}
				catch (Exception e)
				{
				}
			}
		}
	}

	public void bigStep(String string)
	{
		if (string == null)
		{
			_expr = _expr + " -BIG BAD- ";
		}
		else
		{
			_expr = _expr + " -" + string + "- ";
		}
		_main.setStep(_expr);
    if (_stepBig)
    {
			_main.displayContext();
		  synchronized(_thread)
		  {
			  try
  			{
	  			_thread.wait();
		  	}
			  catch (Exception e)
  			{
  			}
      }
		}
	}

	public void setDebugStep(String string)
	{
		if (string.equals("No Stepping"))
		{
			setNoStep();
		}
		else if (string.equals("Big Steps"))
		{
			setBigStep();
		}
		else if (string.equals("Small Steps"))
		{
			setSmallStep();
		}
		else if (string.equals("Baby Steps"))
		{
			setBabyStep();
		}
	}

	public void setBabyStep()
	{
		print("setBabyStep()");
		_stepBig = true;
		_stepSmall = true;
		_stepBaby = true;
	}

	public void setSmallStep()
	{
		print("setSmallStep()");
		_stepBig = true;
		_stepSmall = true;
		_stepBaby = false;
	}

	public void setBigStep()
	{
		print("setBigStep()");
		_stepBig = true;
		_stepSmall = false;
		_stepBaby = false;
	}

	public void setNoStep()
	{
		print("setNoStep()");
		_stepBig = false;
		_stepSmall = false;
		_stepBaby = false;
	}
	
}
