package ShowXPath;

import ShowXPath.XPath.analysis.*;
import ShowXPath.XPath.node.*;

import java.util.*;
import java.io.*;

public class Interpreter extends DepthFirstAdapter
{
	public final static int FORWARD_AXIS = 1;
	public final static int REVERSE_AXIS = 2;
	public final static int NO_AXIS = 3;
	
	private int labelCount = 0;
	private Executor _executor = null;
	private int _axis = FORWARD_AXIS;

	public Interpreter(Executor executor)
	{
		_executor = executor;
	}

	private Interpreter()
	{}

	private void print(String string)
	{
		//System.out.println("In Interpreter: " + string);
	}

	private int genLabel()
	{
		return labelCount++;
	}
	
	public void defaultIn(Node node)
	{
	}

	public void defaultOut(Node node)
	{
	}

	public void inStart(Start node)
	{
		_executor.babyStep("START");
	}

	public void outStart(Start node)
	{
		_executor.babyStep("END");
	}

	public void caseStart(Start node)
	{
		inStart(node);
		_executor.executorStart();
		node.getPExpr().apply(this);
		node.getEOF().apply(this);
		_executor.end();
		outStart(node);
	}

	public void inAExpr(AExpr node)
	{
	}

	public void outAExpr(AExpr node)
	{
	}

	public void caseAExpr(AExpr node)
	{
		inAExpr(node);
		int label = genLabel();
		Object temp[] = node.getOrexprlist().toArray();
		for(int i = 0; i < temp.length; i++)
		{
			((POrexprlist) temp[i]).apply(this);
			_executor.iftrue(Integer.toString(label));
			//_executor.pop();
		}
		if(node.getAndexpr() != null)
		{
			node.getAndexpr().apply(this);
			if (temp.length > 0)
			{
				_executor.label(Integer.toString(label));
			}
		}
		outAExpr(node);
	}

	public void inAAncestorAxisname(AAncestorAxisname node)
	{
	}

	public void outAAncestorAxisname(AAncestorAxisname node)
	{
	}

	public void caseAAncestorAxisname(AAncestorAxisname node)
	{
		inAAncestorAxisname(node);
		if(node.getAncestorTok() != null)
		{
			node.getAncestorTok().apply(this);
			_axis = REVERSE_AXIS;
			_executor.ancestor();
			_executor.smallStep("ancestor");
		}
		outAAncestorAxisname(node);
	}

	public void inAAncestorOrSelfAxisname(AAncestorOrSelfAxisname node)
	{
	}

	public void outAAncestorOrSelfAxisname(AAncestorOrSelfAxisname node)
	{
	}

	public void caseAAncestorOrSelfAxisname(AAncestorOrSelfAxisname node)
	{
		inAAncestorOrSelfAxisname(node);
		if(node.getAncestorOrSelfTok() != null)
		{
			node.getAncestorOrSelfTok().apply(this);
			_axis = REVERSE_AXIS;
			_executor.ancestor_or_self();
			_executor.smallStep("ancestor-or-self");
		}
		outAAncestorOrSelfAxisname(node);
	}

	public void inAAttributeAxisname(AAttributeAxisname node)
	{
	}

	public void outAAttributeAxisname(AAttributeAxisname node)
	{
	}

	public void caseAAttributeAxisname(AAttributeAxisname node)
	{
		inAAttributeAxisname(node);
		if(node.getAttributeTok() != null)
		{
			node.getAttributeTok().apply(this);
			_axis = FORWARD_AXIS;
			_executor.attribute();
			_executor.smallStep("attribute");
		}
		outAAttributeAxisname(node);
	}

	public void inAChildAxisname(AChildAxisname node)
	{
	}

	public void outAChildAxisname(AChildAxisname node)
	{
	}

	public void caseAChildAxisname(AChildAxisname node)
	{
		inAChildAxisname(node);
		if(node.getChildTok() != null)
		{
			node.getChildTok().apply(this);
			_axis = FORWARD_AXIS;
			_executor.child();
			_executor.smallStep("child");
		}
		outAChildAxisname(node);
	}

	public void inADescendentAxisname(ADescendentAxisname node)
	{
	}

	public void outADescendentAxisname(ADescendentAxisname node)
	{
	}

	public void caseADescendentAxisname(ADescendentAxisname node)
	{
		inADescendentAxisname(node);
		if(node.getDescendentTok() != null)
		{
			node.getDescendentTok().apply(this);
			_axis = FORWARD_AXIS;
			_executor.descendent();
			_executor.smallStep("descendent");
		}
		outADescendentAxisname(node);
	}

	public void inADescendentOrSelfAxisname(ADescendentOrSelfAxisname node)
	{
	}

	public void outADescendentOrSelfAxisname(ADescendentOrSelfAxisname node)
	{
	}

	public void caseADescendentOrSelfAxisname(ADescendentOrSelfAxisname node)
	{
		inADescendentOrSelfAxisname(node);
		if(node.getDescendentOrSelfTok() != null)
		{
			node.getDescendentOrSelfTok().apply(this);
			_axis = FORWARD_AXIS;
			_executor.descendent_or_self();
			_executor.smallStep("descendent-or-self");
		}
		outADescendentOrSelfAxisname(node);
	}

	public void inAFollowingAxisname(AFollowingAxisname node)
	{
	}

	public void outAFollowingAxisname(AFollowingAxisname node)
	{
	}

	public void caseAFollowingAxisname(AFollowingAxisname node)
	{
		inAFollowingAxisname(node);
		if(node.getFollowingTok() != null)
		{
			node.getFollowingTok().apply(this);
			_axis = FORWARD_AXIS;
			_executor.following();
			_executor.smallStep("following");
		}
		outAFollowingAxisname(node);
	}

	public void inAFollowingSiblingAxisname(AFollowingSiblingAxisname node)
	{
	}

	public void outAFollowingSiblingAxisname(AFollowingSiblingAxisname node)
	{
	}

	public void caseAFollowingSiblingAxisname(AFollowingSiblingAxisname node)
	{
		inAFollowingSiblingAxisname(node);
		if(node.getFollowingSiblingTok() != null)
		{
			node.getFollowingSiblingTok().apply(this);
			_axis = FORWARD_AXIS;
			_executor.following_sibling();
			_executor.smallStep("following-sibling");
		}
		outAFollowingSiblingAxisname(node);
	}

	public void inANamespaceAxisname(ANamespaceAxisname node)
	{
	}

	public void outANamespaceAxisname(ANamespaceAxisname node)
	{
	}

	public void caseANamespaceAxisname(ANamespaceAxisname node)
	{
		inANamespaceAxisname(node);
		if(node.getNamespaceTok() != null)
		{
			node.getNamespaceTok().apply(this);
			_axis = FORWARD_AXIS;
			_executor.namespace();
			_executor.smallStep("namespace");
		}
		outANamespaceAxisname(node);
	}

	public void inAParentAxisname(AParentAxisname node)
	{
	}

	public void outAParentAxisname(AParentAxisname node)
	{
	}

	public void caseAParentAxisname(AParentAxisname node)
	{
		inAParentAxisname(node);
		if(node.getParentTok() != null)
		{
			node.getParentTok().apply(this);
			_axis = REVERSE_AXIS;
			_executor.parent();
			_executor.smallStep("parent");
		}
		outAParentAxisname(node);
	}

	public void inAPrecedingAxisname(APrecedingAxisname node)
	{
	}

	public void outAPrecedingAxisname(APrecedingAxisname node)
	{
	}

	public void caseAPrecedingAxisname(APrecedingAxisname node)
	{
		inAPrecedingAxisname(node);
		if(node.getPrecedingTok() != null)
		{
			node.getPrecedingTok().apply(this);
			_axis = REVERSE_AXIS;
			_executor.preceding();
			_executor.smallStep("preceding");
		}
		outAPrecedingAxisname(node);
	}

	public void inAPrecedingSiblingAxisname(APrecedingSiblingAxisname node)
	{
	}

	public void outAPrecedingSiblingAxisname(APrecedingSiblingAxisname node)
	{
	}

	public void caseAPrecedingSiblingAxisname(APrecedingSiblingAxisname node)
	{
		inAPrecedingSiblingAxisname(node);
		if(node.getPrecedingSiblingTok() != null)
		{
			node.getPrecedingSiblingTok().apply(this);
			_axis = REVERSE_AXIS;
			_executor.preceding_sibling();
			_executor.smallStep("preceding-sibling");
		}
		outAPrecedingSiblingAxisname(node);
	}

	public void inASelfAxisname(ASelfAxisname node)
	{
	}

	public void outASelfAxisname(ASelfAxisname node)
	{
	}

	public void caseASelfAxisname(ASelfAxisname node)
	{
		inASelfAxisname(node);
		if(node.getSelfTok() != null)
		{
			node.getSelfTok().apply(this);
			_axis = NO_AXIS;
			_executor.self();
			_executor.smallStep("self");
		}
		outASelfAxisname(node);
	}

	public void inANcnameQname(ANcnameQname node)
	{
	}

	public void outANcnameQname(ANcnameQname node)
	{
	}

	public void caseANcnameQname(ANcnameQname node)
	{
		inANcnameQname(node);
		if(node.getNcnameTok() != null)
		{
			node.getNcnameTok().apply(this);
			// Form a qname
			_executor.pushLiteral("");
			_executor.pushLiteral(node.getNcnameTok().getText());
			_executor.qname();
			_executor.noStep(node.getNcnameTok().getText());
		}
		outANcnameQname(node);
	}

	public void inAFullnameQname(AFullnameQname node)
	{
	}

	public void outAFullnameQname(AFullnameQname node)
	{
	}

	public void caseAFullnameQname(AFullnameQname node)
	{
		inAFullnameQname(node);
		if(node.getNamespace() != null)
		{
			node.getNamespace().apply(this);
			_executor.pushLiteral(node.getNamespace().getText());
			_executor.babyStep(node.getNamespace().getText());
		}
		if(node.getColonTok() != null)
		{
			node.getColonTok().apply(this);
			_executor.babyStep(":");
		}
		if(node.getLocalName() != null)
		{
			node.getLocalName().apply(this);
			_executor.pushLiteral(node.getLocalName().getText());
		}
		_executor.qname();
		_executor.noStep(node.getLocalName().getText());

		outAFullnameQname(node);
	}

	public void inAWildcardNametest(AWildcardNametest node)
	{
	}

	public void outAWildcardNametest(AWildcardNametest node)
	{
	}

	public void caseAWildcardNametest(AWildcardNametest node)
	{
		inAWildcardNametest(node);
		if(node.getKleeneTok() != null)
		{
			node.getKleeneTok().apply(this);
			_executor.pushLiteral("");
			_executor.pushLiteral("*");
			_executor.qname();
			_executor.noStep("*");
		}
		outAWildcardNametest(node);
	}

	public void inAQualifiedWildcardNametest(AQualifiedWildcardNametest node)
	{
	}

	public void outAQualifiedWildcardNametest(AQualifiedWildcardNametest node)
	{
	}

	public void caseAQualifiedWildcardNametest(AQualifiedWildcardNametest node)
	{
		inAQualifiedWildcardNametest(node);
		if(node.getNcnameTok() != null)
		{
			node.getNcnameTok().apply(this);
			_executor.pushLiteral(node.getNcnameTok().getText());
			_executor.babyStep(node.getNcnameTok().getText());
		}
		if(node.getColonTok() != null)
		{
			node.getColonTok().apply(this);
			_executor.babyStep(":");
		}
		if(node.getKleeneTok() != null)
		{
			node.getKleeneTok().apply(this);
			_executor.pushLiteral("*");
		}
		_executor.qname();
		_executor.noStep("*");
		outAQualifiedWildcardNametest(node);
	}

	public void inAQnameNametest(AQnameNametest node)
	{
	}

	public void outAQnameNametest(AQnameNametest node)
	{
	}

	public void caseAQnameNametest(AQnameNametest node)
	{
		inAQnameNametest(node);
		if(node.getQname() != null)
		{
			node.getQname().apply(this);
		}
		outAQnameNametest(node);
	}

	public void inANametestNodetest(ANametestNodetest node)
	{
	}

	public void outANametestNodetest(ANametestNodetest node)
	{
	}

	public void caseANametestNodetest(ANametestNodetest node)
	{
		inANametestNodetest(node);
		if(node.getNametest() != null)
		{
			node.getNametest().apply(this);
		}
		_executor.nodetest();
		_executor.smallStep("");
		outANametestNodetest(node);
	}

	public void inACommentNodetest(ACommentNodetest node)
	{
	}

	public void outACommentNodetest(ACommentNodetest node)
	{
	}

	public void caseACommentNodetest(ACommentNodetest node)
	{
		inACommentNodetest(node);
		if(node.getCommentTok() != null)
		{
			node.getCommentTok().apply(this);
		}
		if(node.getRightParenTok() != null)
		{
			node.getRightParenTok().apply(this);
		}
		_executor.comment();
		_executor.smallStep("comment()");
		outACommentNodetest(node);
	}

	public void inATextNodetest(ATextNodetest node)
	{
	}

	public void outATextNodetest(ATextNodetest node)
	{
	}

	public void caseATextNodetest(ATextNodetest node)
	{
		inATextNodetest(node);
		if(node.getTextTok() != null)
		{
			node.getTextTok().apply(this);
		}
		if(node.getRightParenTok() != null)
		{
			node.getRightParenTok().apply(this);
		}
		_executor.text();
		_executor.smallStep("text()");
		outATextNodetest(node);
	}

	public void inANodeNodetest(ANodeNodetest node)
	{
	}

	public void outANodeNodetest(ANodeNodetest node)
	{
	}

	public void caseANodeNodetest(ANodeNodetest node)
	{
		inANodeNodetest(node);
		if(node.getNodeTok() != null)
		{
			node.getNodeTok().apply(this);
		}
		if(node.getRightParenTok() != null)
		{
			node.getRightParenTok().apply(this);
		}
		_executor.smallStep("node()");
		outANodeNodetest(node);
	}

	public void inAProcessingInstructionNodetest(AProcessingInstructionNodetest node)
	{
	}

	public void outAProcessingInstructionNodetest(AProcessingInstructionNodetest node)
	{
	}

	public void caseAProcessingInstructionNodetest(AProcessingInstructionNodetest node)
	{
		inAProcessingInstructionNodetest(node);
		if(node.getProcessingInstructionTok() != null)
		{
			node.getProcessingInstructionTok().apply(this);
			_executor.babyStep("processing-instruction(");
		}
		if(node.getLiteralTok() != null)
		{
			node.getLiteralTok().apply(this);
			_executor.pushLiteral(node.getLiteralTok().getText());
			_executor.babyStep(node.getLiteralTok().getText());
		}
		else
		{
			_executor.mark();
		}
		
		if(node.getRightParenTok() != null)
		{
			node.getRightParenTok().apply(this);
		}
		_executor.processing_instruction();
		_executor.babyStep(")");
		outAProcessingInstructionNodetest(node);
	}

	public void inAFullAxisspecifier(AFullAxisspecifier node)
	{
	}

	public void outAFullAxisspecifier(AFullAxisspecifier node)
	{
	}

	public void caseAFullAxisspecifier(AFullAxisspecifier node)
	{
		inAFullAxisspecifier(node);
		if(node.getAxisname() != null)
		{
			node.getAxisname().apply(this);
		}
		else
		{
			_axis = FORWARD_AXIS;
		}
		
		if(node.getAxisTok() != null)
		{
			node.getAxisTok().apply(this);
			_executor.babyStep("::");
		}
		outAFullAxisspecifier(node);
	}

	public void inAAttributeAxisspecifier(AAttributeAxisspecifier node)
	{
	}

	public void outAAttributeAxisspecifier(AAttributeAxisspecifier node)
	{
	}

	public void caseAAttributeAxisspecifier(AAttributeAxisspecifier node)
	{
		inAAttributeAxisspecifier(node);
		if(node.getAtSignTok() != null)
		{
			node.getAtSignTok().apply(this);
			_executor.attribute();
			_axis = FORWARD_AXIS;
			_executor.smallStep("@");
		}
		outAAttributeAxisspecifier(node);
	}

	public void inALastFunctionname(ALastFunctionname node)
	{
	}

	public void outALastFunctionname(ALastFunctionname node)
	{
	}

	public void caseALastFunctionname(ALastFunctionname node)
	{
		inALastFunctionname(node);
		if(node.getLastTok() != null)
		{
			node.getLastTok().apply(this);
			_executor.pushLiteral( node.getLastTok().getText());
			//_executor.babyStep("last(");
		}
		outALastFunctionname(node);
	}

	public void inAPositionFunctionname(APositionFunctionname node)
	{
	}

	public void outAPositionFunctionname(APositionFunctionname node)
	{
	}

	public void caseAPositionFunctionname(APositionFunctionname node)
	{
		inAPositionFunctionname(node);
		if(node.getPositionTok() != null)
		{
			node.getPositionTok().apply(this);
			_executor.pushLiteral(node.getPositionTok().getText());
			//_executor.babyStep("position(");
		}
		outAPositionFunctionname(node);
	}

	public void inACountFunctionname(ACountFunctionname node)
	{
	}

	public void outACountFunctionname(ACountFunctionname node)
	{
	}

	public void caseACountFunctionname(ACountFunctionname node)
	{
		inACountFunctionname(node);
		if(node.getCountTok() != null)
		{
			node.getCountTok().apply(this);
			_executor.pushLiteral(node.getCountTok().getText());
			//_executor.babyStep("count(");
		}
		outACountFunctionname(node);
	}

	public void inALocalNameFunctionname(ALocalNameFunctionname node)
	{
	}

	public void outALocalNameFunctionname(ALocalNameFunctionname node)
	{
	}

	public void caseALocalNameFunctionname(ALocalNameFunctionname node)
	{
		inALocalNameFunctionname(node);
		if(node.getLocalNameTok() != null)
		{
			node.getLocalNameTok().apply(this);
			_executor.pushLiteral(node.getLocalNameTok().getText());
			//_executor.babyStep("local-name(");
		}
		outALocalNameFunctionname(node);
	}

	public void inANamespaceUriFunctionname(ANamespaceUriFunctionname node)
	{
	}

	public void outANamespaceUriFunctionname(ANamespaceUriFunctionname node)
	{
	}

	public void caseANamespaceUriFunctionname(ANamespaceUriFunctionname node)
	{
		inANamespaceUriFunctionname(node);
		if(node.getNamespaceUriTok() != null)
		{
			node.getNamespaceUriTok().apply(this);
			_executor.pushLiteral(node.getNamespaceUriTok().getText());
			//_executor.babyStep("namespace-uri(");
		}
		outANamespaceUriFunctionname(node);
	}

	public void inAFunctionAvailableTokFunctionname(AFunctionAvailableTokFunctionname node)
	{
	}

	public void outAFunctionAvailableTokFunctionname(AFunctionAvailableTokFunctionname node)
	{
	}

	public void caseAFunctionAvailableTokFunctionname(AFunctionAvailableTokFunctionname node)
	{
		inAFunctionAvailableTokFunctionname(node);
		if(node.getFunctionAvailableTok() != null)
		{
			node.getFunctionAvailableTok().apply(this);
			_executor.pushLiteral(node.getFunctionAvailableTok().getText());
			//_executor.babyStep("function-available(");
		}
		outAFunctionAvailableTokFunctionname(node);
	}

	public void inAElementAvailableTokFunctionname(AElementAvailableTokFunctionname node)
	{
	}

	public void outAElementAvailableTokFunctionname(AElementAvailableTokFunctionname node)
	{
	}

	public void caseAElementAvailableTokFunctionname(AElementAvailableTokFunctionname node)
	{
		inAElementAvailableTokFunctionname(node);
		if(node.getElementAvailableTok() != null)
		{
			node.getElementAvailableTok().apply(this);
			_executor.pushLiteral(node.getElementAvailableTok().getText());
			//_executor.babyStep("element-available(");
		}
		outAElementAvailableTokFunctionname(node);
	}

	public void inAStringFunctionname(AStringFunctionname node)
	{
	}

	public void outAStringFunctionname(AStringFunctionname node)
	{
	}

	public void caseAStringFunctionname(AStringFunctionname node)
	{
		inAStringFunctionname(node);
		if(node.getStringTok() != null)
		{
			node.getStringTok().apply(this);
			_executor.pushLiteral(node.getStringTok().getText());
			//_executor.babyStep("string(");
		}
		outAStringFunctionname(node);
	}

	public void inANameFunctionname(ANameFunctionname node)
	{
	}

	public void outANameFunctionname(ANameFunctionname node)
	{
	}

	public void caseANameFunctionname(ANameFunctionname node)
	{
		inANameFunctionname(node);
		if(node.getNameTok() != null)
		{
			node.getNameTok().apply(this);
			_executor.pushLiteral(node.getNameTok().getText());
			//_executor.babyStep("name(");
		}
		outANameFunctionname(node);
	}

	public void inAConcatFunctionname(AConcatFunctionname node)
	{
	}

	public void outAConcatFunctionname(AConcatFunctionname node)
	{
	}

	public void caseAConcatFunctionname(AConcatFunctionname node)
	{
		inAConcatFunctionname(node);
		if(node.getConcatTok() != null)
		{
			node.getConcatTok().apply(this);
			_executor.pushLiteral(node.getConcatTok().getText());
			//_executor.babyStep("concat(");
		}
		outAConcatFunctionname(node);
	}

	public void inAStartsWithFunctionname(AStartsWithFunctionname node)
	{
	}

	public void outAStartsWithFunctionname(AStartsWithFunctionname node)
	{
	}

	public void caseAStartsWithFunctionname(AStartsWithFunctionname node)
	{
		inAStartsWithFunctionname(node);
		if(node.getStartsWithTok() != null)
		{
			node.getStartsWithTok().apply(this);
			_executor.pushLiteral(node.getStartsWithTok().getText());
			//_executor.babyStep("starts-with(");
		}
		outAStartsWithFunctionname(node);
	}

	public void inAContainsFunctionname(AContainsFunctionname node)
	{
	}

	public void outAContainsFunctionname(AContainsFunctionname node)
	{
	}

	public void caseAContainsFunctionname(AContainsFunctionname node)
	{
		inAContainsFunctionname(node);
		if(node.getContainsTok() != null)
		{
			node.getContainsTok().apply(this);
			_executor.pushLiteral(node.getContainsTok().getText());
			//_executor.babyStep("contains(");
		}
		outAContainsFunctionname(node);
	}

	public void inASubstringBeforeFunctionname(ASubstringBeforeFunctionname node)
	{
	}

	public void outASubstringBeforeFunctionname(ASubstringBeforeFunctionname node)
	{
	}

	public void caseASubstringBeforeFunctionname(ASubstringBeforeFunctionname node)
	{
		inASubstringBeforeFunctionname(node);
		if(node.getSubstringBeforeTok() != null)
		{
			node.getSubstringBeforeTok().apply(this);
			_executor.pushLiteral(node.getSubstringBeforeTok().getText());
			//_executor.babyStep("substring-before(");
		}
		outASubstringBeforeFunctionname(node);
	}

	public void inASubstringAfterFunctionname(ASubstringAfterFunctionname node)
	{
	}

	public void outASubstringAfterFunctionname(ASubstringAfterFunctionname node)
	{
	}

	public void caseASubstringAfterFunctionname(ASubstringAfterFunctionname node)
	{
		inASubstringAfterFunctionname(node);
		if(node.getSubstringAfterTok() != null)
		{
			node.getSubstringAfterTok().apply(this);
			_executor.pushLiteral(node.getSubstringAfterTok().getText());
			//_executor.babyStep("substring-after(");
		}
		outASubstringAfterFunctionname(node);
	}

	public void inASubstringFunctionname(ASubstringFunctionname node)
	{
	}

	public void outASubstringFunctionname(ASubstringFunctionname node)
	{
	}

	public void caseASubstringFunctionname(ASubstringFunctionname node)
	{
		inASubstringFunctionname(node);
		if(node.getSubstringTok() != null)
		{
			node.getSubstringTok().apply(this);
			_executor.pushLiteral(node.getSubstringTok().getText());
			//_executor.babyStep("subtring(");
		}
		outASubstringFunctionname(node);
	}

	public void inAStringLengthFunctionname(AStringLengthFunctionname node)
	{
	}

	public void outAStringLengthFunctionname(AStringLengthFunctionname node)
	{
	}

	public void caseAStringLengthFunctionname(AStringLengthFunctionname node)
	{
		inAStringLengthFunctionname(node);
		if(node.getStringLengthTok() != null)
		{
			node.getStringLengthTok().apply(this);
			_executor.pushLiteral(node.getStringLengthTok().getText());
			//_executor.babyStep("string-length(");
		}
		outAStringLengthFunctionname(node);
	}

	public void inANormalizeSpaceFunctionname(ANormalizeSpaceFunctionname node)
	{
	}

	public void outANormalizeSpaceFunctionname(ANormalizeSpaceFunctionname node)
	{
	}

	public void caseANormalizeSpaceFunctionname(ANormalizeSpaceFunctionname node)
	{
		inANormalizeSpaceFunctionname(node);
		if(node.getNormalizeSpaceTok() != null)
		{
			node.getNormalizeSpaceTok().apply(this);
			_executor.pushLiteral(node.getNormalizeSpaceTok().getText());
			//_executor.babyStep("normalize-space(");
		}
		outANormalizeSpaceFunctionname(node);
	}

	public void inATranslateFunctionname(ATranslateFunctionname node)
	{
	}

	public void outATranslateFunctionname(ATranslateFunctionname node)
	{
	}

	public void caseATranslateFunctionname(ATranslateFunctionname node)
	{
		inATranslateFunctionname(node);
		if(node.getTranslateTok() != null)
		{
			node.getTranslateTok().apply(this);
			_executor.pushLiteral(node.getTranslateTok().getText());
			//_executor.babyStep("translate(");
		}
		outATranslateFunctionname(node);
	}

	public void inABooleanFunctionname(ABooleanFunctionname node)
	{
	}

	public void outABooleanFunctionname(ABooleanFunctionname node)
	{
	}

	public void caseABooleanFunctionname(ABooleanFunctionname node)
	{
		inABooleanFunctionname(node);
		if(node.getBooleanTok() != null)
		{
			node.getBooleanTok().apply(this);
			_executor.pushLiteral(node.getBooleanTok().getText());
			//_executor.babyStep("boolean(");
		}
		outABooleanFunctionname(node);
	}

	public void inANotFunctionname(ANotFunctionname node)
	{
	}

	public void outANotFunctionname(ANotFunctionname node)
	{
	}

	public void caseANotFunctionname(ANotFunctionname node)
	{
		inANotFunctionname(node);
		if(node.getNotTok() != null)
		{
			node.getNotTok().apply(this);
			_executor.pushLiteral(node.getNotTok().getText());
			//_executor.babyStep("not(");
		}
		outANotFunctionname(node);
	}

	public void inATrueFunctionname(ATrueFunctionname node)
	{
	}

	public void outATrueFunctionname(ATrueFunctionname node)
	{
	}

	public void caseATrueFunctionname(ATrueFunctionname node)
	{
		inATrueFunctionname(node);
		if(node.getTrueTok() != null)
		{
			node.getTrueTok().apply(this);
			_executor.pushLiteral(node.getTrueTok().getText());
			//_executor.babyStep("true(");
		}
		outATrueFunctionname(node);
	}

	public void inAFalseFunctionname(AFalseFunctionname node)
	{
	}

	public void outAFalseFunctionname(AFalseFunctionname node)
	{
	}

	public void caseAFalseFunctionname(AFalseFunctionname node)
	{
		inAFalseFunctionname(node);
		if(node.getFalseTok() != null)
		{
			node.getFalseTok().apply(this);
			_executor.pushLiteral(node.getFalseTok().getText());
			//_executor.babyStep("false(");
		}
		outAFalseFunctionname(node);
	}

	public void inALangFunctionname(ALangFunctionname node)
	{
	}

	public void outALangFunctionname(ALangFunctionname node)
	{
	}

	public void caseALangFunctionname(ALangFunctionname node)
	{
		inALangFunctionname(node);
		if(node.getLangTok() != null)
		{
			node.getLangTok().apply(this);
			_executor.pushLiteral(node.getLangTok().getText());
			//_executor.babyStep("lang(");
		}
		outALangFunctionname(node);
	}

	public void inANumberStringFunctionname(ANumberStringFunctionname node)
	{
	}

	public void outANumberStringFunctionname(ANumberStringFunctionname node)
	{
	}

	public void caseANumberStringFunctionname(ANumberStringFunctionname node)
	{
		inANumberStringFunctionname(node);
		if(node.getNumberStringTok() != null)
		{
			node.getNumberStringTok().apply(this);
			_executor.pushLiteral(node.getNumberStringTok().getText());
			//_executor.babyStep("number(");
		}
		outANumberStringFunctionname(node);
	}

	public void inASumFunctionname(ASumFunctionname node)
	{
	}

	public void outASumFunctionname(ASumFunctionname node)
	{
	}

	public void caseASumFunctionname(ASumFunctionname node)
	{
		inASumFunctionname(node);
		if(node.getSumTok() != null)
		{
			node.getSumTok().apply(this);
			_executor.pushLiteral(node.getSumTok().getText());
			//_executor.babyStep("sum(");
		}
		outASumFunctionname(node);
	}

	public void inAFloorFunctionname(AFloorFunctionname node)
	{
	}

	public void outAFloorFunctionname(AFloorFunctionname node)
	{
	}

	public void caseAFloorFunctionname(AFloorFunctionname node)
	{
		inAFloorFunctionname(node);
		if(node.getFloorTok() != null)
		{
			node.getFloorTok().apply(this);
			_executor.pushLiteral(node.getFloorTok().getText());
			//_executor.babyStep("floor(");
		}
		outAFloorFunctionname(node);
	}

	public void inACeilingFunctionname(ACeilingFunctionname node)
	{
	}

	public void outACeilingFunctionname(ACeilingFunctionname node)
	{
	}

	public void caseACeilingFunctionname(ACeilingFunctionname node)
	{
		inACeilingFunctionname(node);
		if(node.getCeilingTok() != null)
		{
			node.getCeilingTok().apply(this);
			_executor.pushLiteral(node.getCeilingTok().getText());
			//_executor.babyStep("ceiling(");
		}
		outACeilingFunctionname(node);
	}

	public void inARoundFunctionname(ARoundFunctionname node)
	{
	}

	public void outARoundFunctionname(ARoundFunctionname node)
	{
	}

	public void caseARoundFunctionname(ARoundFunctionname node)
	{
		inARoundFunctionname(node);
		if(node.getRoundTok() != null)
		{
			node.getRoundTok().apply(this);
			_executor.pushLiteral(node.getRoundTok().getText());
			//_executor.babyStep("round(");
		}
		outARoundFunctionname(node);
	}

	public void inADocumentFunctionname(ADocumentFunctionname node)
	{
	}

	public void outADocumentFunctionname(ADocumentFunctionname node)
	{
	}

	public void caseADocumentFunctionname(ADocumentFunctionname node)
	{
		inADocumentFunctionname(node);
		if(node.getDocumentTok() != null)
		{
			node.getDocumentTok().apply(this);
			_executor.pushLiteral(node.getDocumentTok().getText());
			//_executor.babyStep("document(");
		}
		outADocumentFunctionname(node);
	}

	public void inAKeyFunctionname(AKeyFunctionname node)
	{
	}

	public void outAKeyFunctionname(AKeyFunctionname node)
	{
	}

	public void caseAKeyFunctionname(AKeyFunctionname node)
	{
		inAKeyFunctionname(node);
		if(node.getKeyTok() != null)
		{
			node.getKeyTok().apply(this);
			_executor.pushLiteral(node.getKeyTok().getText());
			//_executor.babyStep("key(");
		}
		outAKeyFunctionname(node);
	}

	public void inAIdFunctionname(AIdFunctionname node)
	{
	}

	public void outAIdFunctionname(AIdFunctionname node)
	{
	}

	public void caseAIdFunctionname(AIdFunctionname node)
	{
		inAIdFunctionname(node);
		if(node.getIdTok() != null)
		{
			node.getIdTok().apply(this);
			_executor.pushLiteral(node.getIdTok().getText());
			//_executor.babyStep("id(");
		}
		outAIdFunctionname(node);
	}

	public void inAFormatNumberFunctionname(AFormatNumberFunctionname node)
	{
	}

	public void outAFormatNumberFunctionname(AFormatNumberFunctionname node)
	{
	}

	public void caseAFormatNumberFunctionname(AFormatNumberFunctionname node)
	{
		inAFormatNumberFunctionname(node);
		if(node.getFormatNumberTok() != null)
		{
			node.getFormatNumberTok().apply(this);
			_executor.pushLiteral(node.getFormatNumberTok().getText());
			//_executor.babyStep("format-number(");
		}
		outAFormatNumberFunctionname(node);
	}

	public void inACurrentFunctionname(ACurrentFunctionname node)
	{
	}

	public void outACurrentFunctionname(ACurrentFunctionname node)
	{
	}

	public void caseACurrentFunctionname(ACurrentFunctionname node)
	{
		inACurrentFunctionname(node);
		if(node.getCurrentTok() != null)
		{
			node.getCurrentTok().apply(this);
			_executor.pushLiteral(node.getCurrentTok().getText());
			//_executor.babyStep("current(");
		}
		outACurrentFunctionname(node);
	}

	public void inAUnparsedEntityUriFunctionname(AUnparsedEntityUriFunctionname node)
	{
	}

	public void outAUnparsedEntityUriFunctionname(AUnparsedEntityUriFunctionname node)
	{
	}

	public void caseAUnparsedEntityUriFunctionname(AUnparsedEntityUriFunctionname node)
	{
		inAUnparsedEntityUriFunctionname(node);
		if(node.getUnparsedEntityUriTok() != null)
		{
			node.getUnparsedEntityUriTok().apply(this);
			_executor.pushLiteral(node.getUnparsedEntityUriTok().getText());
			//_executor.babyStep("unparsed-entity-URI(");
		}
		outAUnparsedEntityUriFunctionname(node);
	}

	public void inAGenerateIdFunctionname(AGenerateIdFunctionname node)
	{
	}

	public void outAGenerateIdFunctionname(AGenerateIdFunctionname node)
	{
	}

	public void caseAGenerateIdFunctionname(AGenerateIdFunctionname node)
	{
		inAGenerateIdFunctionname(node);
		if(node.getGenerateIdTok() != null)
		{
			node.getGenerateIdTok().apply(this);
			_executor.pushLiteral(node.getGenerateIdTok().getText());
			//_executor.babyStep("generate-id(");
		}
		outAGenerateIdFunctionname(node);
	}

	public void inASystemPropertyFunctionname(ASystemPropertyFunctionname node)
	{
	}

	public void outASystemPropertyFunctionname(ASystemPropertyFunctionname node)
	{
	}

	public void caseASystemPropertyFunctionname(ASystemPropertyFunctionname node)
	{
		inASystemPropertyFunctionname(node);
		if(node.getSystemPropertyTok() != null)
		{
			node.getSystemPropertyTok().apply(this);
			_executor.pushLiteral(node.getSystemPropertyTok().getText());
			//_executor.babyStep("system-property(");
		}
		outASystemPropertyFunctionname(node);
	}

	public void inAExprlist(AExprlist node)
	{
	}

	public void outAExprlist(AExprlist node)
	{
	}

	public void caseAExprlist(AExprlist node)
	{
		inAExprlist(node);
		if(node.getExpr() != null)
		{
			node.getExpr().apply(this);
		}
		if(node.getCommaTok() != null)
		{
			node.getCommaTok().apply(this);
			_executor.babyStep(",");
		}
		outAExprlist(node);
	}

	public void inAParameterlist(AParameterlist node)
	{
	}

	public void outAParameterlist(AParameterlist node)
	{
	}

	public void caseAParameterlist(AParameterlist node)
	{
		inAParameterlist(node);
		{
			Object temp[] = node.getExprlist().toArray();
			for(int i = 0; i < temp.length; i++)
			{
				((PExprlist) temp[i]).apply(this);
			}
		}
		if(node.getExpr() != null)
		{
			node.getExpr().apply(this);
		}
		outAParameterlist(node);
	}

	public void inAVariableFilterexpr(AVariableFilterexpr node)
	{
	}

	public void outAVariableFilterexpr(AVariableFilterexpr node)
	{
	}

	public void caseAVariableFilterexpr(AVariableFilterexpr node)
	{
		inAVariableFilterexpr(node);
		if(node.getDollarTok() != null)
		{
			node.getDollarTok().apply(this);
		}
		if(node.getQname() != null)
		{
			node.getQname().apply(this);
			_executor.value();
		}
		_executor.babyStep("$");
		outAVariableFilterexpr(node);
	}

	public void inAParethesisExprFilterexpr(AParethesisExprFilterexpr node)
	{
	}

	public void outAParethesisExprFilterexpr(AParethesisExprFilterexpr node)
	{
	}

	public void caseAParethesisExprFilterexpr(AParethesisExprFilterexpr node)
	{
		inAParethesisExprFilterexpr(node);
		if(node.getLeftParenTok() != null)
		{
			node.getLeftParenTok().apply(this);
			_executor.babyStep("(");
		}
		if(node.getExpr() != null)
		{
			node.getExpr().apply(this);
		}
		if(node.getRightParenTok() != null)
		{
			node.getRightParenTok().apply(this);
			_executor.babyStep(")");
		}
		outAParethesisExprFilterexpr(node);
	}

	public void inALiteralFilterexpr(ALiteralFilterexpr node)
	{
	}

	public void outALiteralFilterexpr(ALiteralFilterexpr node)
	{
	}

	public void caseALiteralFilterexpr(ALiteralFilterexpr node)
	{
		inALiteralFilterexpr(node);
		if(node.getLiteralTok() != null)
		{
			node.getLiteralTok().apply(this);
			_executor.pushLiteral(node.getLiteralTok().getText());
			_executor.babyStep(node.getLiteralTok().getText());
		}
		outALiteralFilterexpr(node);
	}

	public void inANumberFilterexpr(ANumberFilterexpr node)
	{
	}

	public void outANumberFilterexpr(ANumberFilterexpr node)
	{
	}

	public void caseANumberFilterexpr(ANumberFilterexpr node)
	{
		inANumberFilterexpr(node);
		if(node.getNumberTok() != null)
		{
			node.getNumberTok().apply(this);
			_executor.pushNumber(node.getNumberTok().getText());
			_executor.babyStep(node.getNumberTok().getText());
		}
		outANumberFilterexpr(node);
	}

	public void inAFunctioncallFilterexpr(AFunctioncallFilterexpr node)
	{
	}

	public void outAFunctioncallFilterexpr(AFunctioncallFilterexpr node)
	{
	}

	public void caseAFunctioncallFilterexpr(AFunctioncallFilterexpr node)
	{
		inAFunctioncallFilterexpr(node);
		_executor.mark();
		if(node.getFunctionname() != null)
		{
			_executor.babyStep(node.getFunctionname().toString());
		}
		if(node.getParameterlist() != null)
		{
			node.getParameterlist().apply(this);
		}
		if(node.getRightParenTok() != null)
		{
			node.getRightParenTok().apply(this);
		}
		if(node.getFunctionname() != null)
		{
			node.getFunctionname().apply(this);
		}
		_executor.call();
		_executor.babyStep(")");
		outAFunctioncallFilterexpr(node);
	}

	public void inARootPathexpr(ARootPathexpr node)
	{
	}

	public void outARootPathexpr(ARootPathexpr node)
	{
		//_executor.showContext();
	}

	public void caseARootPathexpr(ARootPathexpr node)
	{
		inARootPathexpr(node);
		if(node.getSlashTok() != null)
		{
			node.getSlashTok().apply(this);
			_executor.root();
			_executor.bigStep("/");
		}
		outARootPathexpr(node);
	}

	public void inARelativePathexpr(ARelativePathexpr node)
	{
	}

	public void outARelativePathexpr(ARelativePathexpr node)
	{
		//_executor.showContext();
	}

	public void caseARelativePathexpr(ARelativePathexpr node)
	{
		inARelativePathexpr(node);
		_executor.context();
		_executor.bigStep("current()/");
		if(node.getStep() != null)
		{
			node.getStep().apply(this);
		}
		{
			Object temp[] = node.getNextStep().toArray();
			for(int i = 0; i < temp.length; i++)
			{
				((PNextStep) temp[i]).apply(this);
			}
		}
		outARelativePathexpr(node);
	}

	public void inAAbsolutePathexpr(AAbsolutePathexpr node)
	{
	}

	public void outAAbsolutePathexpr(AAbsolutePathexpr node)
	{
		//_executor.showContext();
	}

	public void caseAAbsolutePathexpr(AAbsolutePathexpr node)
	{
		inAAbsolutePathexpr(node);
		//_executor.bigStep("/");
		_executor.root();
		{
			Object temp[] = node.getNextStep().toArray();
			for(int i = 0; i < temp.length; i++)
			{
				((PNextStep) temp[i]).apply(this);
			}
		}
		outAAbsolutePathexpr(node);
	}

	public void inAFilterFromRootPathPathexpr(AFilterFromRootPathPathexpr node)
	{
	}

	public void outAFilterFromRootPathPathexpr(AFilterFromRootPathPathexpr node)
	{
		//_executor.showContext();
	}

	public void caseAFilterFromRootPathPathexpr(AFilterFromRootPathPathexpr node)
	{
		inAFilterFromRootPathPathexpr(node);
		if(node.getFilterexpr() != null)
		{
			node.getFilterexpr().apply(this);
		}
		{
			Object temp[] = node.getPredicate().toArray();
			_executor.setForwardAxis();
			for(int i = 0; i < temp.length; i++)
			{
				_executor.prepare();
				_executor.smallStep("");
				((PPredicate) temp[i]).apply(this);
				_executor.update();
				_executor.smallStep("");
			}
		}
		{
			Object temp[] = node.getNextStep().toArray();
			for(int i = 0; i < temp.length; i++)
			{
				((PNextStep) temp[i]).apply(this);
			}
		}
		outAFilterFromRootPathPathexpr(node);
	}

	public void inAPredicate(APredicate node)
	{
	}

	public void outAPredicate(APredicate node)
	{
	}

	public void caseAPredicate(APredicate node)
	{
		inAPredicate(node);
		if(node.getLeftSquareTok() != null)
		{
			node.getLeftSquareTok().apply(this);
			_executor.babyStep("[");
		}
		if(node.getExpr() != null)
		{
			node.getExpr().apply(this);
		}
		if(node.getRightSquareTok() != null)
		{
			node.getRightSquareTok().apply(this);
			_executor.babyStep("]");
		}
		outAPredicate(node);
	}

	public void inADescendentNextStep(ADescendentNextStep node)
	{
	}

	public void outADescendentNextStep(ADescendentNextStep node)
	{
	}

	public void caseADescendentNextStep(ADescendentNextStep node)
	{
		inADescendentNextStep(node);
		if(node.getDoubleslashTok() != null)
		{
			node.getDoubleslashTok().apply(this);
			_executor.descendent_or_self();
			_executor.bigStep("//");
		}
		if(node.getStep() != null)
		{
			node.getStep().apply(this);
		}
		outADescendentNextStep(node);
	}

	public void inAChildNextStep(AChildNextStep node)
	{
	}

	public void outAChildNextStep(AChildNextStep node)
	{
	}

	public void caseAChildNextStep(AChildNextStep node)
	{
		inAChildNextStep(node);
		if(node.getSlashTok() != null)
		{
			node.getSlashTok().apply(this);
			_executor.bigStep("/");
		}
		if(node.getStep() != null)
		{
			node.getStep().apply(this);
		}
		outAChildNextStep(node);
	}

	public void inAFullStep(AFullStep node)
	{
	}

	public void outAFullStep(AFullStep node)
	{
		//_executor.showContext();
	}

	public void caseAFullStep(AFullStep node)
	{
		inAFullStep(node);
		if(node.getAxisspecifier() != null)
		{
			node.getAxisspecifier().apply(this);
		}
		else
		{
			_executor.child();
			_executor.smallStep("child");
			_executor.babyStep("::");
		}
		if(node.getNodetest() != null)
		{
			node.getNodetest().apply(this);
		}
		{
			Object temp[] = node.getPredicate().toArray();
			if (_axis == FORWARD_AXIS)
			{
				_executor.setForwardAxis();
			}
			else
			{
				_executor.setReverseAxis();
			}
			for(int i = 0; i < temp.length; i++)
			{
				_executor.prepare();
				_executor.smallStep("");
				((PPredicate) temp[i]).apply(this);
				_executor.update();
				_executor.smallStep("");
			}
		}
		outAFullStep(node);
	}

	public void inACurrentStep(ACurrentStep node)
	{
	}

	public void outACurrentStep(ACurrentStep node)
	{
		//_executor.showContext();
	}

	public void caseACurrentStep(ACurrentStep node)
	{
		inACurrentStep(node);
		if(node.getDotTok() != null)
		{
			node.getDotTok().apply(this);
			_executor.bigStep(".");
		}
		outACurrentStep(node);
	}

	public void inAParentStep(AParentStep node)
	{
	}

	public void outAParentStep(AParentStep node)
	{
		//_executor.showContext();
	}

	public void caseAParentStep(AParentStep node)
	{
		inAParentStep(node);
		if(node.getDoubledotTok() != null)
		{
			node.getDoubledotTok().apply(this);
			_executor.parent();
			_executor.bigStep("..");
		}
		outAParentStep(node);
	}

	public void inAAdditionAddexprlist(AAdditionAddexprlist node)
	{
	}

	public void outAAdditionAddexprlist(AAdditionAddexprlist node)
	{
	}

	public void caseAAdditionAddexprlist(AAdditionAddexprlist node)
	{
		inAAdditionAddexprlist(node);
		if(node.getMulexpr() != null)
		{
			node.getMulexpr().apply(this);
		}
		if(node.getPlusTok() != null)
		{
			node.getPlusTok().apply(this);
			_executor.babyStep("+");
		}
		outAAdditionAddexprlist(node);
	}

	public void inASubtractionAddexprlist(ASubtractionAddexprlist node)
	{
	}

	public void outASubtractionAddexprlist(ASubtractionAddexprlist node)
	{
	}

	public void caseASubtractionAddexprlist(ASubtractionAddexprlist node)
	{
		inASubtractionAddexprlist(node);
		if(node.getMulexpr() != null)
		{
			node.getMulexpr().apply(this);
		}
		if(node.getMinusTok() != null)
		{
			node.getMinusTok().apply(this);
			_executor.babyStep("-");
		}
		outASubtractionAddexprlist(node);
	}

	public void inAAddexpr(AAddexpr node)
	{
	}

	public void outAAddexpr(AAddexpr node)
	{
	}

	public void caseAAddexpr(AAddexpr node)
	{
		inAAddexpr(node);
		Object temp[] = node.getAddexprlist().toArray();
		final int PLUS = 0;
		final int MINUS = 1;
		int op = PLUS;
		for(int i = 0; i < temp.length; i++)
		{
			((PAddexprlist) temp[i]).apply(this);
			if (i != 0)
			{
				if (op == PLUS)
				{
					_executor.add();
				}
				else
				{
					_executor.sub();
				}
			}
			if (temp[i] instanceof AAdditionAddexprlist)
			{
				op = PLUS;
			}
			else
			{
				op = MINUS;
			}
		}
		if(node.getMulexpr() != null)
		{
			node.getMulexpr().apply(this);
		}
		if (temp.length > 0)
		{
			if (op == PLUS)
			{
				_executor.add();
			}
			else
			{
				_executor.sub();
			}
		}
		outAAddexpr(node);
	}

	public void inAAndexprlist(AAndexprlist node)
	{
	}

	public void outAAndexprlist(AAndexprlist node)
	{
	}

	public void caseAAndexprlist(AAndexprlist node)
	{
		inAAndexprlist(node);
		if(node.getEqexpr() != null)
		{
			node.getEqexpr().apply(this);
		}
		if(node.getAndTok() != null)
		{
			node.getAndTok().apply(this);
			_executor.babyStep(" and ");
		}
		outAAndexprlist(node);
	}

	public void inAAndexpr(AAndexpr node)
	{
	}

	public void outAAndexpr(AAndexpr node)
	{
	}

	public void caseAAndexpr(AAndexpr node)
	{
		inAAndexpr(node);
		int label = genLabel();
		Object temp[] = node.getAndexprlist().toArray();
		for(int i = 0; i < temp.length; i++)
		{
			((PAndexprlist) temp[i]).apply(this);
			_executor.iffalse(Integer.toString(label));
			//_executor.pop();
		}
		if(node.getEqexpr() != null)
		{
			node.getEqexpr().apply(this);
			if (temp.length > 0)
			{
				_executor.label(Integer.toString(label));
			}
		}
		outAAndexpr(node);
	}

	public void inAEqualEqexprlist(AEqualEqexprlist node)
	{
	}

	public void outAEqualEqexprlist(AEqualEqexprlist node)
	{
	}

	public void caseAEqualEqexprlist(AEqualEqexprlist node)
	{
		inAEqualEqexprlist(node);
		if(node.getRelationalexpr() != null)
		{
			node.getRelationalexpr().apply(this);
		}
		if(node.getEqualTok() != null)
		{
			node.getEqualTok().apply(this);
			_executor.babyStep("=");
		}
		outAEqualEqexprlist(node);
	}

	public void inANotequalEqexprlist(ANotequalEqexprlist node)
	{
	}

	public void outANotequalEqexprlist(ANotequalEqexprlist node)
	{
	}

	public void caseANotequalEqexprlist(ANotequalEqexprlist node)
	{
		inANotequalEqexprlist(node);
		if(node.getRelationalexpr() != null)
		{
			node.getRelationalexpr().apply(this);
		}
		if(node.getNotEqualTok() != null)
		{
			node.getNotEqualTok().apply(this);
			_executor.babyStep("!=");
		}
		outANotequalEqexprlist(node);
	}

	public void inAEqexpr(AEqexpr node)
	{
	}

	public void outAEqexpr(AEqexpr node)
	{
	}

	public void caseAEqexpr(AEqexpr node)
	{
		inAEqexpr(node);
		Object temp[] = node.getEqexprlist().toArray();
		final int EQUAL = 0;
		final int NOT_EQUAL = 1;
		int op = EQUAL;
		for(int i = 0; i < temp.length; i++)
		{
			((PEqexprlist) temp[i]).apply(this);
			if (i != 0)
			{
				if (op == EQUAL)
				{
					_executor.equal();
				}
				else
				{
					_executor.equal();
          _executor.not();
				}
			}
			if (temp[i] instanceof AEqualEqexprlist)
			{
				op = EQUAL;
			}
			else
			{
				op = NOT_EQUAL;
			}
		}
		if(node.getRelationalexpr() != null)
		{
			node.getRelationalexpr().apply(this);
		}
		if (temp.length > 0)
		{
			if (op == EQUAL)
			{
				_executor.equal();
			}
			else
			{
				_executor.equal();
				_executor.not();
			}
		}
		outAEqexpr(node);
	}

	public void inAMulMulexprlist(AMulMulexprlist node)
	{
	}

	public void outAMulMulexprlist(AMulMulexprlist node)
	{
	}

	public void caseAMulMulexprlist(AMulMulexprlist node)
	{
		inAMulMulexprlist(node);
		if(node.getUnaryexpr() != null)
		{
			node.getUnaryexpr().apply(this);
		}
		if(node.getStarTok() != null)
		{
			node.getStarTok().apply(this);
			_executor.babyStep("*");
		}
		outAMulMulexprlist(node);
	}

	public void inADivMulexprlist(ADivMulexprlist node)
	{
	}

	public void outADivMulexprlist(ADivMulexprlist node)
	{
	}

	public void caseADivMulexprlist(ADivMulexprlist node)
	{
		inADivMulexprlist(node);
		if(node.getUnaryexpr() != null)
		{
			node.getUnaryexpr().apply(this);
		}
		if(node.getDivTok() != null)
		{
			node.getDivTok().apply(this);
			_executor.babyStep(" div ");
		}
		outADivMulexprlist(node);
	}

	public void inAModMulexprlist(AModMulexprlist node)
	{
	}

	public void outAModMulexprlist(AModMulexprlist node)
	{
	}

	public void caseAModMulexprlist(AModMulexprlist node)
	{
		inAModMulexprlist(node);
		if(node.getUnaryexpr() != null)
		{
			node.getUnaryexpr().apply(this);
		}
		if(node.getModTok() != null)
		{
			node.getModTok().apply(this);
			_executor.babyStep(" mod ");
		}
		outAModMulexprlist(node);
	}

	public void inAMulexpr(AMulexpr node)
	{
	}

	public void outAMulexpr(AMulexpr node)
	{
	}

	public void caseAMulexpr(AMulexpr node)
	{
		inAMulexpr(node);
		Object temp[] = node.getMulexprlist().toArray();
		final int MUL = 0;
		final int DIV = 1;
		final int MOD = 2;
		int op = MUL;
		for(int i = 0; i < temp.length; i++)
		{
			((PMulexprlist) temp[i]).apply(this);
			if (i != 0)
			{
				if (op == MUL)
				{
					_executor.mul();
				}
				else if (op == DIV)
				{
					_executor.div();
				}
				else
				{
          _executor.mod();
				}
			}
			if (temp[i] instanceof AMulMulexprlist)
			{
				op = MUL;
			}
			else if (temp[i] instanceof ADivMulexprlist)
			{
				op = DIV;
			}
			else
			{
				op = MOD;
			}
		}
		if(node.getUnaryexpr() != null)
		{
			node.getUnaryexpr().apply(this);
		}
		if (temp.length > 0)
		{
			if (op == MUL)
			{
				_executor.mul();
			}
			else if (op == DIV)
			{
				_executor.div();
			}
			else 
			{
				_executor.mod();
			}
		}
		outAMulexpr(node);
	}

	public void inAOrexprlist(AOrexprlist node)
	{
	}

	public void outAOrexprlist(AOrexprlist node)
	{
	}

	public void caseAOrexprlist(AOrexprlist node)
	{
		inAOrexprlist(node);
		if(node.getAndexpr() != null)
		{
			node.getAndexpr().apply(this);
		}
		if(node.getOrTok() != null)
		{
			node.getOrTok().apply(this);
			_executor.babyStep(" or ");
		}
		outAOrexprlist(node);
	}

	public void inALessRelationalexprlist(ALessRelationalexprlist node)
	{
	}

	public void outALessRelationalexprlist(ALessRelationalexprlist node)
	{
	}

	public void caseALessRelationalexprlist(ALessRelationalexprlist node)
	{
		inALessRelationalexprlist(node);
		if(node.getAddexpr() != null)
		{
			node.getAddexpr().apply(this);
		}
		if(node.getLessTok() != null)
		{
			node.getLessTok().apply(this);
			_executor.babyStep("<");
		}
		outALessRelationalexprlist(node);
	}

	public void inAGreaterRelationalexprlist(AGreaterRelationalexprlist node)
	{
	}

	public void outAGreaterRelationalexprlist(AGreaterRelationalexprlist node)
	{
	}

	public void caseAGreaterRelationalexprlist(AGreaterRelationalexprlist node)
	{
		inAGreaterRelationalexprlist(node);
		if(node.getAddexpr() != null)
		{
			node.getAddexpr().apply(this);
		}
		if(node.getGreaterTok() != null)
		{
			node.getGreaterTok().apply(this);
			_executor.babyStep(">");
		}
		outAGreaterRelationalexprlist(node);
	}

	public void inALessEqualRelationalexprlist(ALessEqualRelationalexprlist node)
	{
	}

	public void outALessEqualRelationalexprlist(ALessEqualRelationalexprlist node)
	{
	}

	public void caseALessEqualRelationalexprlist(ALessEqualRelationalexprlist node)
	{
		inALessEqualRelationalexprlist(node);
		if(node.getAddexpr() != null)
		{
			node.getAddexpr().apply(this);
		}
		if(node.getLessEqualTok() != null)
		{
			node.getLessEqualTok().apply(this);
			_executor.babyStep("<=");
		}
		outALessEqualRelationalexprlist(node);
	}

	public void inAGreaterEqualRelationalexprlist(AGreaterEqualRelationalexprlist node)
	{
	}

	public void outAGreaterEqualRelationalexprlist(AGreaterEqualRelationalexprlist node)
	{
	}

	public void caseAGreaterEqualRelationalexprlist(AGreaterEqualRelationalexprlist node)
	{
		inAGreaterEqualRelationalexprlist(node);
		if(node.getAddexpr() != null)
		{
			node.getAddexpr().apply(this);
		}
		if(node.getGreaterEqualTok() != null)
		{
			node.getGreaterEqualTok().apply(this);
			_executor.babyStep(">=");
		}
		outAGreaterEqualRelationalexprlist(node);
	}

	public void inARelationalexpr(ARelationalexpr node)
	{
	}

	public void outARelationalexpr(ARelationalexpr node)
	{
	}

	public void caseARelationalexpr(ARelationalexpr node)
	{
		inARelationalexpr(node);
		final int LESS = 0;
		final int GREATER = 1;
		final int LESS_EQUAL = 2;
		final int GREATER_EQUAL = 3;
		int op = LESS;
		Object temp[] = node.getRelationalexprlist().toArray();
		for(int i = 0; i < temp.length; i++)
		{
			((PRelationalexprlist) temp[i]).apply(this);
			if (i != 0)
			{
				if (op == LESS)
				{
					_executor.less();
				}
				else if (op == GREATER)
				{
					_executor.greater();
				}
				else if (op == LESS_EQUAL)
				{
					_executor.less_equal();
				}
				else
				{
					_executor.greater_equal();
				}
			}
			if (temp[i] instanceof ALessRelationalexprlist)
			{
				op = LESS;
			}
			else if (temp[i] instanceof ALessEqualRelationalexprlist)
			{
				op = LESS_EQUAL;
			}
			else if (temp[i] instanceof AGreaterRelationalexprlist)
			{
				op = GREATER;
			}
			else
			{
				op = GREATER_EQUAL;
			}
		}
		if(node.getAddexpr() != null)
		{
			node.getAddexpr().apply(this);
		}
		if (temp.length > 0)
		{
			if (op == LESS)
			{
				_executor.less();
			}
			else if (op == GREATER)
			{
				_executor.greater();
			}
			else if (op == LESS_EQUAL)
			{
				_executor.less_equal();
			}
			else
			{
				_executor.greater_equal();
			}
		}
		outARelationalexpr(node);
	}

	public void inAUnaryexpr(AUnaryexpr node)
	{
	}

	public void outAUnaryexpr(AUnaryexpr node)
	{
	}

	public void caseAUnaryexpr(AUnaryexpr node)
	{
		inAUnaryexpr(node);
		if(node.getUnionexpr() != null)
		{
			node.getUnionexpr().apply(this);
		}
		Object temp[] = node.getMinusTok().toArray();
		for(int i = 0; i < temp.length; i++)
		{
			((TMinusTok) temp[i]).apply(this);
			_executor.negate();
			_executor.babyStep("-");
		}
		/*
			if(node.getUnionexpr() != null)
			{
			node.getUnionexpr().apply(this);
			}
		*/
		outAUnaryexpr(node);
	}

	public void inAUnionexprlist(AUnionexprlist node)
	{
	}

	public void outAUnionexprlist(AUnionexprlist node)
	{
	}

	public void caseAUnionexprlist(AUnionexprlist node)
	{
		inAUnionexprlist(node);
		if(node.getPathexpr() != null)
		{
			_executor.startPath();
			node.getPathexpr().apply(this);
			_executor.endPath();
		}
		if(node.getBarTok() != null)
		{
			node.getBarTok().apply(this);
			_executor.babyStep("|");
		}
		outAUnionexprlist(node);
	}

	public void inAUnionexpr(AUnionexpr node)
	{
	}

	public void outAUnionexpr(AUnionexpr node)
	{
	}

	public void caseAUnionexpr(AUnionexpr node)
	{
		inAUnionexpr(node);
		Object temp[] = node.getUnionexprlist().toArray();
		for(int i = 0; i < temp.length; i++)
		{
			((PUnionexprlist) temp[i]).apply(this);
			if (i != 0)
			{
				_executor.union();
			}
		}
		if(node.getPathexpr() != null)
		{
			_executor.startPath();
			node.getPathexpr().apply(this);
			_executor.endPath();
		}
		if (temp.length > 0)
		{
			_executor.union();
		}
		outAUnionexpr(node);
	}
}
